@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* 
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later
         and under FreeBSD license revised, i.e. without advertising clause.
 */
@@ -448,7 +448,7 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.channel = channel_no;
 	out.lun = lun_no;
 
-	out.devname = burn_strdup(fname);
+	out.devname = strdup(fname);
 
 	out.cam = NULL;
 	out.lock_fd = -1;
@@ -477,7 +477,7 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.read_toc = mmc_read_toc;
 	out.write = mmc_write;
 	out.erase = mmc_erase;
-	out.read_sectors = mmc_read_sectors;
+	out.read_cd = mmc_read_cd;
 	out.perform_opc = mmc_perform_opc;
 	out.set_speed = mmc_set_speed;
 	out.send_parameters = spc_select_error_params;
@@ -492,13 +492,13 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.idata = calloc(1, sizeof(struct burn_scsi_inquiry_data));
 	out.idata->valid = 0;
 	out.mdata = calloc(1, sizeof(struct scsi_mode_data));
-	out.mdata->valid = 0;
 	if (out.idata == NULL || out.mdata == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1, 0x00020108,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 			"Could not allocate new drive object", 0, 0);
 		return;
 	}
+	out.mdata->p2a_valid = 0;
 	memset(&out.params, 0, sizeof(struct params));
 	t = burn_drive_register(&out);
 
@@ -861,8 +861,11 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 
 		memset(&ccb->csio.sense_data, 0, sizeof(ccb->csio.sense_data));
 		memset(c->sense, 0, sizeof(c->sense));
+		c->start_time = burn_get_time(0);
+
 		err = cam_send_ccb(d->cam, ccb);
 
+		c->end_time = burn_get_time(0);
 		ignore_error = sense_len = 0;
 		/* ts B00325 : CAM_AUTOSNS_VALID advised by Alexander Motin */
 		if (ccb->ccb_h.status & CAM_AUTOSNS_VALID) {
@@ -959,19 +962,12 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				c->sense[13] = 0x00;
 				done = 1;
 			}
-
-			/* >>> Need own duration time measurement.
-			       Then remove bit1 from flag.
-			*/
-			done = scsi_eval_cmd_outcome(d, c, fp, c->sense,
-						sense_len, 0, start_time,
-						timeout_ms, i,
-						2 | !!ignore_error);
-			if (d->cancel)
-				done = 1;
-		} else {
-			done = 1;
 		}
+		done = scsi_eval_cmd_outcome(d, c, fp, c->sense,
+						sense_len, start_time,
+						timeout_ms, i, !!ignore_error);
+		if (d->cancel)
+			done = 1;
 	} while (!done);
 	ret = 1;
 ex:;
@@ -1055,7 +1051,7 @@ int burn_os_is_2k_seekrw(char *path, int flag)
                   0 = could not estimate size capacity of file object
                   1 = estimation has been made, bytes was set
 */
-int burn_os_stdio_capacity(char *path, off_t *bytes)
+int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 {
 	struct stat stbuf;
 	struct statvfs vfsbuf;
@@ -1108,7 +1104,7 @@ int burn_os_stdio_capacity(char *path, off_t *bytes)
 			{ret = -2; goto ex;}
 		*bytes = add_size;
 	} else if(S_ISREG(stbuf.st_mode)) {
-		add_size = stbuf.st_blocks * (off_t) 512;
+		add_size = burn_sparse_file_addsize(write_start, &stbuf);
 		strcpy(testpath, path);
 	} else
 		{ret = 0; goto ex;}
