@@ -1,6 +1,6 @@
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -15,6 +15,12 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <time.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 
 /* ts A80914 : This is unneeded. Version info comes from libburn.h.
 #include "v ersion.h"
@@ -328,3 +334,68 @@ char *burn_printify(char *msg)
 	return msg;
 }
 
+
+/* ts B30521 */
+void burn_int_to_lsb(int val, char *target)
+{
+	unsigned char *buf;
+
+	buf = (unsigned char *) target;
+	buf[0] = val & 0xff;
+	buf[1] = (val >> 8) & 0xff;
+	buf[2] = (val >> 16) & 0xff;
+	buf[3] = (val >> 24) & 0xff;
+}
+
+
+/* ts B30609 */
+double burn_get_time(int flag)
+{
+	int ret;
+	struct timeval tv;
+
+#ifdef Libburn_use_clock_gettime_monotoniC
+#ifdef _POSIX_TIMERS
+#ifdef _POSIX_MONOTONIC_CLOCK
+
+	/* Enable by
+	     export CFLAGS=-DLibburn_use_clock_gettime_monotoniC
+	     export LIBS=-lrt
+	     ./configure ... && make clean && make
+	*/
+
+	struct timespec tp;
+	ret = clock_gettime(CLOCK_MONOTONIC, &tp);
+	if (ret == 0)
+		return ((double) tp.tv_sec) + ((double) tp.tv_nsec) * 1.0e-9;
+
+#endif /* _POSIX_MONOTONIC_CLOCK */
+#endif /* _POSIX_TIMERS */
+#endif /* Xorriso_use_clock_gettime_monotoniC */
+
+	ret = gettimeofday(&tv, NULL);
+	if (ret == 0)
+		return ((double) tv.tv_sec) + ((double) tv.tv_usec) * 1.0e-6;
+	return (double) time(NULL);
+}
+
+/* ts B40609 */
+off_t burn_sparse_file_addsize(off_t write_start, struct stat *stbuf)
+{
+	off_t add_size;
+
+	add_size = stbuf->st_blocks * (off_t) 512;
+	if (add_size < stbuf->st_size) {
+		/* Sparse file */
+		if (write_start < stbuf->st_size) {
+			/* Might write into sparse gaps */
+			if (write_start > add_size)
+				add_size = write_start;
+		} else {
+			/* Will not write into sparse area */
+			add_size = stbuf->st_size;
+		}
+	}
+	return add_size;
+}
+
