@@ -86,6 +86,10 @@ extern struct libdax_msgs *libdax_messenger;
 */
 #define Libburn_bd_streamed_obS (64 * 1024)
 
+/* The number of retries if write(2) returns a short, non-negative write count.
+*/
+#define Libburn_stdio_write_retrieS 16
+
 
 static int type_to_ctrl(int mode)
 {
@@ -1101,7 +1105,7 @@ int burn_write_track(struct burn_write_opts *o, struct burn_session *s,
 	struct burn_drive *d = o->drive;
 	int i, tmp = 0, open_ended = 0, ret= 0, nwa, lba;
 	int sectors;
-	char msg[80];
+	char msg[160];
 
 	d->rlba = -150;
 
@@ -2575,29 +2579,41 @@ int burn_stdio_read_source(struct burn_source *source, char *buf, int bufsize,
 int burn_stdio_write(int fd, char *buf, int count, struct burn_drive *d, 
 			 int flag)
 {
-	int ret;
+	int ret = 0;
 	char *msg = NULL;
+	int todo, done, retries;
 
-	if (d->cancel)
+	if (d->cancel || count <= 0)
 		return 0;
+
+	todo = count;
+	done = 0;
+	for (retries = 0; todo > 0 && retries <= Libburn_stdio_write_retrieS;
+             retries++) {
 /*
 fprintf(stderr, "libburn_DEBUG: write(%d, %lX, %d)\n",
                 fd, (unsigned long) buf, count);
 */
-
-	ret = write(fd, buf, count);
-	if (ret != count) {
+		ret = write(fd, buf + done, todo);
+		if (ret < 0)
+	break;
+		done += ret;
+		todo -= ret;
+	}
+	if (done != count) {
 		BURN_ALLOC_MEM(msg, char, 160);
 
-		sprintf(msg,
-		  "Cannot write desired amount of data. write(2) returned %d.",
-		        ret);
+		sprintf(msg, "Cannot write desired amount of %d bytes.", count);
+		if (retries > 1)
+			sprintf(msg + strlen(msg), " Did %d retries. Last",
+			        retries - 1);
+		sprintf(msg + strlen(msg), " write(2) returned %d.", ret);
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00020148,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			msg, errno, 0);
 		d->cancel = 1;
-		return 0;
+		ret = 0; goto ex;
 	}
 ex:;
 	BURN_FREE_MEM(msg);
@@ -2662,10 +2678,11 @@ int burn_stdio_mmc_dummy_write(struct burn_drive *d, int start,
 /* ts A70911 */
 /* Flush stdio system buffer to physical device.
    @param flag bit0= do not report debug message (intermediate sync)
+               bit1= do fsync(2) unconditionally
 */
 int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 {
-	int ret;
+	int ret, do_fsync;
 	char *msg = NULL;
 
 	if (fd < 0) {
@@ -2678,11 +2695,20 @@ int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 		return 0;
 	}
 	d->needs_sync_cache = 0;
-	if (!(flag & 1))
-		libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
-			   LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
-			   "syncing cache (stdio fsync)", 0, 0);
-	ret = fsync(fd);
+	do_fsync = 0;
+	if (flag & 2)
+		do_fsync = 1;
+	else if (d->write_opts != NULL)
+		do_fsync = (d->write_opts->stdio_fsync_size >= 0);
+	if (do_fsync) {
+		if (!(flag & 1))
+			libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
+				"syncing cache (stdio fsync)", 0, 0);
+		ret = fsync(fd);
+	} else {
+		ret = 0;
+	}
 	if (ret != 0 && errno == EIO) {
 		BURN_ALLOC_MEM(msg, char, 160);
 
@@ -3283,7 +3309,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		if(d->drive_role == 1)
 			d->sync_cache(d);
 		else
-			burn_stdio_sync_cache(fd, d, 0);
+			burn_stdio_sync_cache(fd, d, 2);
 		d->needs_sync_cache = 0;
 	}
 		
