@@ -166,6 +166,9 @@ struct isoburn {
 
  /* Forwarding an image generation option to the burn wrapper */
  int do_tao;
+
+ /* Forwarding an image generation option to the burn wrapper */
+ int do_fsync;
  
 };
 
@@ -529,6 +532,11 @@ struct isoburn_imgen_opts {
      */
     unsigned int joliet_long_names :1;
 
+    /**
+     * Use UTF-16BE rather than its subset UCS-2
+     */
+    unsigned int joliet_utf16 :1;
+
     /**
      * Store timestamps as GMT rather than in local time.
      */
@@ -700,6 +708,9 @@ struct isoburn_imgen_opts {
      */
     int do_tao;
 
+    /* Whether to fsync() stdio_drives after isoburn_activate_session() */
+    int do_fsync;
+
 };
 
 
