@@ -375,7 +375,7 @@ int isoburn_attach_start_lba(struct burn_drive *d, int lba, int flag)
 */
 int isoburn_activate_session(struct burn_drive *drive)
 {
- int ret;
+ int ret, do_sync = 1;
  struct isoburn *o;
 
  ret = isoburn_find_emulator(&o, drive, 0);
@@ -391,9 +391,12 @@ int isoburn_activate_session(struct burn_drive *drive)
        (o->fabricated_disc_status == BURN_DISC_BLANK &&
         o->zero_nwa > 0)))
    return 1;
- 
+ ret = burn_drive_get_drive_role(drive);
+ if (ret != 1)
+   do_sync = !! o->do_fsync;
+
  ret = burn_random_access_write(drive, (off_t) 0, (char*)o->target_iso_head, 
-                                o->target_iso_head_size, 1);
+                                o->target_iso_head_size, do_sync);
 
  return ret;
 }
