@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2012 Thomas Schmitt
+ * Copyright (c) 2009 - 2013 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -87,6 +87,12 @@
 #define ISO_GPT_ENTRIES_MAX 248
 
 
+/* How many warnings to issue about writing Joliet names which cannot be
+   properly represented in UCS-2 and thus had to be defaultet to '_'.
+*/
+#define ISO_JOLIET_UCS2_WARN_MAX 3
+
+
 /**
  * Holds the options for the image generation.
  */
@@ -94,7 +100,7 @@ struct iso_write_opts {
 
     int will_cancel;
 
-    int level; /**< ISO level to write at. (ECMA-119, 10) */
+    int iso_level; /**< ISO level to write at. (ECMA-119, 10) */
 
     /** Which extensions to support. */
     unsigned int rockridge :1;
@@ -192,6 +198,11 @@ struct iso_write_opts {
      */
     unsigned int joliet_long_names :1;
 
+    /**
+     * Use UTF-16BE rather than its subset UCS-2
+     */
+    unsigned int joliet_utf16 :1;
+
     /**
      * Write Rock Ridge info as of specification RRIP-1.10 rather than
      * RRIP-1.12: signature "RRIP_1991A" rather than "IEEE_1282",
@@ -420,6 +431,7 @@ struct iso_write_opts {
      * See ecma119_image : System Area related information
      */
     char *system_area_data;
+    int system_area_size;
     int system_area_options;
 
     /* User settable PVD time stamps */
@@ -502,70 +514,19 @@ struct ecma119_image
     IsoImage *image;
     Ecma119Node *root;
 
-    int will_cancel :1;
+    IsoWriteOpts *opts;
 
-    unsigned int iso_level :2;
-
-    /* extensions */
-    unsigned int rockridge :1;
-    unsigned int joliet :1;
+    /** Whether El Torito data will be produced */
     unsigned int eltorito :1;
-    unsigned int iso1999 :1;
-    unsigned int hfsplus :1;
-    unsigned int fat :1;
 
-    unsigned int hardlinks:1; /* see iso_write_opts_set_hardlinks() */
-
-    unsigned int aaip :1;     /* see iso_write_opts_set_aaip() */
-
-    /* allways write timestamps in GMT */
-    unsigned int always_gmt :1;
-
-    /* relaxed constraints */
-    unsigned int allow_dir_id_ext :1;
-    unsigned int omit_version_numbers :2;
-    unsigned int allow_deep_paths :1;
-    unsigned int allow_longer_paths :1;
-    unsigned int max_37_char_filenames :1;
-    unsigned int no_force_dots :2;
-    unsigned int allow_lowercase :1;
-    unsigned int allow_full_ascii :1;
-    unsigned int allow_7bit_ascii :1;
-
-    unsigned int relaxed_vol_atts : 1;
-
-    /** Allow paths on Joliet tree to be larger than 240 bytes */
-    unsigned int joliet_longer_paths :1;
-
-    /** Allow Joliet names up to 103 characters rather than 64  */
-    unsigned int joliet_long_names :1;
-
-    /** Write old fashioned RRIP-1.10 rather than RRIP-1.12 */
-    unsigned int rrip_version_1_10 :1;
-
-    /** Write field PX with file serial number even with RRIP-1.10 */
-    unsigned int rrip_1_10_px_ino :1;
-
-    /* Write AAIP as extension according to SUSP 1.10 rather than SUSP 1.12. */
-    unsigned int aaip_susp_1_10 :1;
-
-    /* Store in ECMA-119, Joliet, ISO 9660:1999 timestamp the mtime of source
-       bit0= ECMA-119, bit1= Joliet, bit2= ISO 9660:1999.
+    /* The ECMA-119 directory node where to store Rock Ridge relocated
+       directories. (Path is in IsoWriteOpts.rr_reloc_dir)
     */
-    unsigned int dir_rec_mtime :3;
-
-    /* The ECMA-119 directory where to store Rock Ridge relocated directories.
-    */
-    char *rr_reloc_dir;          /* IsoNode name in root directory */
-    int rr_reloc_flags; 
     Ecma119Node *rr_reloc_node;  /* Directory node in ecma119_image */
 
-    unsigned int md5_session_checksum :1;
-    unsigned int md5_file_checksums :2;
-
     /*
      * Mode replace. If one of these flags is set, the correspodent values are
-     * replaced with values below.
+     * replaced with values below. Both get computed from IsoWriteOpts.
      */
     unsigned int replace_uid :1;
     unsigned int replace_gid :1;
@@ -573,30 +534,17 @@ struct ecma119_image
     unsigned int replace_dir_mode :1;
     unsigned int replace_timestamps :1;
 
+    /* Mode replacement values. */ 
     uid_t uid;
     gid_t gid;
     mode_t file_mode;
     mode_t dir_mode;
     time_t timestamp;
 
-    unsigned int old_empty :1;
-    unsigned int untranslated_name_len;
-
-    /**
-     *  if sort files or not. Sorting is based of the weight of each file
-     */
-    int sort_files;
-
+    /* Effective charsets */
     char *input_charset;
     char *output_charset;
 
-    /* See iso_write_opts and iso_write_opts_set_hfsp_serial_number().
-     * 00...00 means that it shall be generated by libisofs.
-     */
-    uint8_t hfsp_serial_number[8];
-
-    unsigned int appendable : 1;
-    uint32_t ms_block; /**< start block for a ms image */
     time_t now; /**< Time at which writing began. */
 
     /** Total size of the output. This only includes the current volume. */
@@ -642,6 +590,7 @@ struct ecma119_image
     uint32_t joliet_path_table_size;
     uint32_t joliet_l_path_table_pos;
     uint32_t joliet_m_path_table_pos;
+    size_t joliet_ucs2_failures;
 
     /*
      * HFS+ related information
@@ -763,14 +712,6 @@ struct ecma119_image
                              Use only underneath ecma119_image_new()
                              and if not NULL*/
 
-    /* ??? Is there a reason why we copy lots of items from IsoWriteOpts
-           rather than taking ownership of the IsoWriteOpts object which
-           is submitted with ecma119_image_new() ?
-     */
-
-    char scdbackup_tag_parm[100];
-    char *scdbackup_tag_written;
-
     /* Buffer for communication between burn_source and writer thread */
     IsoRingBuffer *buffer;
 
@@ -779,20 +720,7 @@ struct ecma119_image
     int wthread_is_running;
     pthread_attr_t th_attr;
 
-    /* User settable PVD time stamps */
-    time_t vol_creation_time;
-    time_t vol_modification_time;
-    time_t vol_expiration_time;
-    time_t vol_effective_time;
-    /* To eventually override vol_creation_time and vol_modification_time
-     * by unconverted string with timezone 0
-     */
-    char vol_uuid[17];
-
-    /* The number of unclaimed 2K blocks before
-       start of partition 1 as of the MBR in system area. */
-    uint32_t partition_offset;
-    /* Partition table parameter: 1 to 63, 0= disabled/default */
+    /* Effective partition table parameter: 1 to 63, 0= disabled/default */
     int partition_secs_per_head;
     /* 1 to 255, 0= disabled/default */
     int partition_heads_per_cyl;
@@ -813,27 +741,22 @@ struct ecma119_image
     uint32_t j_part_l_path_table_pos;
     uint32_t j_part_m_path_table_pos;
 
-#ifdef Libisofs_with_libjtE
-    struct libjte_env *libjte_handle;
-#endif /* Libisofs_with_libjtE */
-
-    uint32_t tail_blocks;
-
     /* Memorized ELF parameters from MIPS Little Endian boot file */
     uint32_t mipsel_e_entry;
     uint32_t mipsel_p_offset;
     uint32_t mipsel_p_vaddr;
     uint32_t mipsel_p_filesz;
 
-    char *appended_partitions[ISO_MAX_PARTITIONS]; 
-    uint8_t appended_part_types[ISO_MAX_PARTITIONS];
+    /* A data file of which the position and size shall be written after
+       a SUN Disk Label.
+    */
+    IsoFileSrc *sparc_core_src;
+
     /* Counted in blocks of 2048 */
     uint32_t appended_part_prepad[ISO_MAX_PARTITIONS];
     uint32_t appended_part_start[ISO_MAX_PARTITIONS];
     uint32_t appended_part_size[ISO_MAX_PARTITIONS];
 
-    char ascii_disc_label[ISO_DISC_LABEL_SIZE];
-
     /* See IsoImage and libisofs.h */
     IsoNode *hfsplus_blessed[ISO_HFSPLUS_BLESS_MAX];
 
@@ -841,17 +764,12 @@ struct ecma119_image
        Only change a block size if it is 0. Set only to 512 or 2048.
        If it stays 0 then it will become 512 or 2048 in time.
     */
-    /* Blocksize of Apple Partition Map
-       May be defined to 512 or 2048 before writer thread starts.
-    */
-    int apm_block_size;
 
     /* Allocation block size of HFS+
        May be defined to 512 or 2048 before hfsplus_writer_create().
     */
-    int hfsp_block_size;
-    int hfsp_cat_node_size; /* 2 * apm_block_size */
-    int hfsp_iso_block_fac; /* 2048 / apm_block_size */
+    int hfsp_cat_node_size; /* 2 * hfsp_block_size */
+    int hfsp_iso_block_fac; /* 2048 / hfsp_block_size */
 
     /* Apple Partition Map description. To be composed during IsoImageWriter
        method ->compute_data_blocks() by calling iso_register_apm_entry().
@@ -872,7 +790,6 @@ struct ecma119_image
     struct iso_mbr_partition_request *mbr_req[ISO_MBR_ENTRIES_MAX];
     int mbr_req_count;
 
-    char *prep_partition; 
     uint32_t prep_part_size;
 
     /* GPT description. To be composed during IsoImageWriter
@@ -885,13 +802,14 @@ struct ecma119_image
     /* bit0= GPT partitions may overlap */
     int gpt_req_flags;
 
-    char *efi_boot_partition;
     uint32_t efi_boot_part_size;
     IsoFileSrc *efi_boot_part_filesrc; /* Just a pointer. Do not free. */
 
     /* Messages from gpt_tail_writer_compute_data_blocks() to
        iso_write_system_area().
     */
+    uint8_t gpt_disk_guid[16];
+    int gpt_disk_guid_set;
     /* Start of GPT entries in System Area, block size 512 */
     uint32_t gpt_part_start;
     /* The ISO block number after the backup GPT header , block size 2048 */
@@ -1049,4 +967,7 @@ void ecma119_set_voldescr_times(IsoImageWriter *writer,
 int iso_write_partition_file(Ecma119Image *target, char *path,
                              uint32_t prepad, uint32_t blocks, int flag);
 
+void issue_ucs2_warning_summary(size_t failures);
+
+
 #endif /*LIBISO_ECMA119_H_*/
