@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2010 - 2013 Thomas Schmitt
+ * Copyright (c) 2010 - 2014 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -82,15 +82,16 @@ void el_torito_set_load_seg(ElToritoBootImage *bootimg, short segment)
 {
     if (bootimg->type != 0)
         return;
-    bootimg->load_seg = segment;
+    if (segment < 0)
+        bootimg->load_seg = 0x1000 + segment;
+    else
+        bootimg->load_seg = segment;
 }
 
 /* API */
 int el_torito_get_load_seg(ElToritoBootImage *bootimg)
 {
-   if (bootimg->load_seg < 0)
-       return 0xffff - bootimg->load_seg;
-   return bootimg->load_seg;
+   return (int) bootimg->load_seg;
 }
  
 /**
@@ -102,15 +103,16 @@ void el_torito_set_load_size(ElToritoBootImage *bootimg, short sectors)
 {
     if (bootimg->type != 0)
         return;
-    bootimg->load_size = sectors;
+    if (sectors < 0)
+        bootimg->load_size = 0x10000 + sectors;
+    else
+        bootimg->load_size = sectors;
 }
 
 /* API */
 int el_torito_get_load_size(ElToritoBootImage *bootimg)
 {
-   if (bootimg->load_size < 0)
-       return 0xffff - bootimg->load_size;
-   return bootimg->load_size;
+   return (int) bootimg->load_size;
 }
 
 /**
@@ -440,6 +442,8 @@ int create_image(IsoImage *image, const char *image_path,
     iso_node_ref(imgfile); /* get our ref */
     boot->bootable = 1;
     boot->seems_boot_info_table = 0;
+    boot->seems_grub2_boot_info = 0;
+    boot->seems_isohybrid_capable = 0;
     boot->isolinux_options = 0;
     boot->type = boot_media_type;
     boot->partition_type = partition_type;
@@ -951,7 +955,8 @@ int catalog_is_repeatable(IsoStream *stream)
 /**
  * fs_id will be the id reserved for El-Torito
  * dev_id will be 0 for catalog, 1 for boot image (if needed)
- * we leave ino_id for future use when we support multiple boot images
+ * ino_id 0 is supposed to be unique. At write time it will get assigned
+ * an automatic file serial number in the ISO, if needed.
  */
 static
 void catalog_get_id(IsoStream *stream, unsigned int *fs_id, dev_t *dev_id,
