@@ -115,8 +115,8 @@ uint8_t get_class (uint16_t v)
   return hfsplus_class_pages[high][low];
 }
 
-static
-int set_hfsplus_name(Ecma119Image *t, char *name, HFSPlusNode *node)
+int iso_get_hfsplus_name(char *input_charset, int imgid, char *name,
+                  uint16_t **result, uint32_t *result_len, uint16_t **cmp_name)
 {
     int ret;
     uint16_t *ucs_name, *iptr, *optr;
@@ -128,19 +128,19 @@ int set_hfsplus_name(Ecma119Image *t, char *name, HFSPlusNode *node)
         return ISO_SUCCESS;
     }
 
-    ret = str2ucs(t->input_charset, name, &ucs_name);
+    ret = str2utf16be(input_charset, name, &ucs_name);
     if (ret < 0) {
-        iso_msg_debug(t->image->id, "Can't convert %s", name);
+        iso_msg_debug(imgid, "Cannot convert '%s'", name);
         return ret;
     }
 
     curlen = ucslen (ucs_name);
-    node->name = calloc ((curlen * HFSPLUS_MAX_DECOMPOSE_LEN + 1),
-			 sizeof (node->name[0]));
-    if (!node->name)
-      return ISO_OUT_OF_MEM;
+    *result = calloc ((curlen * HFSPLUS_MAX_DECOMPOSE_LEN + 1),
+                     sizeof (uint16_t));
+    if (*result == NULL)
+        return ISO_OUT_OF_MEM;
 
-    for (iptr = ucs_name, optr = node->name; *iptr; iptr++)
+    for (iptr = ucs_name, optr = *result; *iptr; iptr++)
       {
 	const uint16_t *dptr;
 	uint16_t val = iso_ntohs (*iptr);
@@ -189,7 +189,7 @@ int set_hfsplus_name(Ecma119Image *t, char *name, HFSPlusNode *node)
 	if (!ucs_name[0])
 	  break;
 	last_class = get_class (ucs_name[0]);
-	for (optr = node->name + 1; *optr; optr++)
+	for (optr = *result + 1; *optr; optr++)
 	  {
 	    uint8_t new_class = get_class (*optr);
 
@@ -207,11 +207,11 @@ int set_hfsplus_name(Ecma119Image *t, char *name, HFSPlusNode *node)
       }
     while (done);
 
-    node->cmp_name = calloc ((ucslen (node->name) + 1), sizeof (node->cmp_name[0]));
-    if (!node->cmp_name)
+    *cmp_name = calloc ((ucslen (*result) + 1), sizeof (uint16_t));
+    if (*cmp_name == NULL)
       return ISO_OUT_OF_MEM;
 
-    for (iptr = node->name, optr = node->cmp_name; *iptr; iptr++)
+    for (iptr = *result, optr = *cmp_name; *iptr; iptr++)
       {
 	*optr = iso_hfsplus_cichar(*iptr);
 	if (*optr != 0)
@@ -221,10 +221,19 @@ int set_hfsplus_name(Ecma119Image *t, char *name, HFSPlusNode *node)
 
     free (ucs_name);
 
-    node->strlen = ucslen (node->name);
+    *result_len = ucslen (*result);
     return ISO_SUCCESS;
 }
 
+static
+int set_hfsplus_name(Ecma119Image *t, char *name, HFSPlusNode *node)
+{
+    int ret;
+
+    ret = iso_get_hfsplus_name(t->input_charset, t->image->id, name,
+                            &(node->name), &(node->strlen), &(node->cmp_name));
+    return ret;
+}
 
 /* >>> ts B20617
        This should be HFSPlusNode rather than IsoNode in order to have access
@@ -429,7 +438,7 @@ int hfsplus_tail_writer_compute_data_blocks(IsoImageWriter *writer)
   }
 
   t = writer->target;
-  block_size = t->hfsp_block_size;
+  block_size = t->opts->hfsp_block_size;
   block_fac = t->hfsp_iso_block_fac;
 
 #ifdef Libisofs_ts_debuG
@@ -466,7 +475,8 @@ int hfsplus_tail_writer_compute_data_blocks(IsoImageWriter *writer)
 
   t->hfsp_total_blocks = hfsp_curblock - t->hfsp_part_start;
 
-  return iso_quick_apm_entry(t, t->hfsp_part_start / block_fac,
+  return iso_quick_apm_entry(t->apm_req, &(t->apm_req_count),
+                             t->hfsp_part_start / block_fac,
 			     t->hfsp_total_blocks / block_fac +
 			     !!(t->hfsp_total_blocks % block_fac),
                             "HFSPLUS_Hybrid", "Apple_HFS");
@@ -477,17 +487,16 @@ static
 int hfsplus_writer_compute_data_blocks(IsoImageWriter *writer)
 {
     Ecma119Image *t;
-    uint32_t i, link_blocks, hfsp_curblock;
-    uint32_t block_fac, cat_node_size, block_size;
+    uint32_t i, hfsp_curblock;
+    uint32_t block_fac, block_size;
 
     if (writer == NULL) {
         return ISO_OUT_OF_MEM;
     }
 
     t = writer->target;
-    block_size = t->hfsp_block_size;
+    block_size = t->opts->hfsp_block_size;
     block_fac = t->hfsp_iso_block_fac;
-    cat_node_size = t->hfsp_cat_node_size;
 
     iso_msg_debug(t->image->id, "(b) curblock=%d, nodes =%d", t->curblock, t->hfsp_nnodes);
     t->hfsp_part_start = t->curblock * block_fac;
@@ -500,7 +509,7 @@ int hfsplus_writer_compute_data_blocks(IsoImageWriter *writer)
     t->hfsp_catalog_file_start = hfsp_curblock;
 
 /* 
-    hfsp_curblock += (t->hfsp_nnodes * cat_node_size + block_size - 1) / block_size;
+    hfsp_curblock += (t->hfsp_nnodes * t->hfsp_cat_node_size + block_size - 1) / block_size;
 */
     hfsp_curblock += 2 * t->hfsp_nnodes;
 
@@ -509,7 +518,6 @@ int hfsplus_writer_compute_data_blocks(IsoImageWriter *writer)
 
     iso_msg_debug(t->image->id, "(d) hfsp_curblock=%d, nodes =%d", hfsp_curblock, t->hfsp_nnodes);
 
-    link_blocks = 0;
     for (i = 0; i < t->hfsp_nleafs; i++)
       if (t->hfsp_leafs[i].unix_type == UNIX_SYMLINK)
 	{
@@ -584,7 +592,7 @@ write_sb (Ecma119Image *t)
 
     iso_msg_debug(t->image->id, "Write HFS+ superblock");
 
-    block_size = t->hfsp_block_size;
+    block_size = t->opts->hfsp_block_size;
 
     memset (buffer, 0, sizeof (buffer));
     ret = iso_write(t, buffer, 1024);
@@ -644,7 +652,7 @@ write_sb (Ecma119Image *t)
 
     }
 
-    memcpy (&sb.num_serial, &t->hfsp_serial_number, 8);
+    memcpy (&sb.num_serial, &t->opts->hfsp_serial_number, 8);
     ret = iso_write(t, &sb, sizeof (sb));
     if (ret < 0)
       return ret;
@@ -667,7 +675,7 @@ int hfsplus_writer_write_data(IsoImageWriter *writer)
     }
 
     t = writer->target;
-    block_size = t->hfsp_block_size;
+    block_size = t->opts->hfsp_block_size;
     block_fac = t->hfsp_iso_block_fac;
     cat_node_size = t->hfsp_cat_node_size;
 
@@ -1035,7 +1043,7 @@ int hfsplus_tail_writer_write_data(IsoImageWriter *writer)
     }
 
     t = writer->target;
-    block_size = t->hfsp_block_size;
+    block_size = t->opts->hfsp_block_size;
 
 #ifdef Libisofs_ts_debuG
     iso_msg_debug(t->image->id, "hfsplus tail writer writes at = %.f",
@@ -1193,7 +1201,7 @@ int subst_symlink_dest_comp(Ecma119Image *target, uint32_t idx,
     wpt += *comp_start - *dest;
     memcpy(wpt, new_name, new_len);
     wpt += new_len;
-    if (*comp_end - *dest < *dest_len)
+    if ((unsigned int) (*comp_end - *dest) < *dest_len)
         memcpy(wpt, *comp_end, *dest_len - (*comp_end - *dest));
     wpt += *dest_len - (*comp_end - *dest);
     *wpt = 0;
@@ -1227,7 +1235,7 @@ int update_symlink(Ecma119Image *target, uint32_t changed_idx, char *new_name,
     char *orig_dest, *orig_start, *orig_end;
     char *hfsp_dest, *hfsp_start, *hfsp_end;
     int ret = 0;
-    unsigned int comp_len, orig_len, hfsp_len, hfsp_comp_len;
+    unsigned int comp_len, orig_len, hfsp_len;
 
     if (target->hfsp_leafs[link_idx].node->type != LIBISO_SYMLINK)
         return ISO_SUCCESS;
@@ -1271,7 +1279,6 @@ int update_symlink(Ecma119Image *target, uint32_t changed_idx, char *new_name,
         hfsp_end = strchr(hfsp_start, '/');
         if (hfsp_end == NULL)
             hfsp_end = hfsp_start + strlen(hfsp_start);
-        hfsp_comp_len = hfsp_end - hfsp_start;
 
         if (comp_len == 0 || (comp_len == 1 && orig_start[0] == '.'))
    continue;
@@ -1582,10 +1589,10 @@ int hfsplus_writer_create(Ecma119Image *target)
     make_hfsplus_decompose_pages();
     make_hfsplus_class_pages();
 
-    if (target->hfsp_block_size == 0)
-        target->hfsp_block_size = HFSPLUS_DEFAULT_BLOCK_SIZE;
-    target->hfsp_cat_node_size = 2 * target->hfsp_block_size;
-    target->hfsp_iso_block_fac = 2048 / target->hfsp_block_size;
+    if (target->opts->hfsp_block_size == 0)
+        target->opts->hfsp_block_size = HFSPLUS_DEFAULT_BLOCK_SIZE;
+    target->hfsp_cat_node_size = 2 * target->opts->hfsp_block_size;
+    target->hfsp_iso_block_fac = 2048 / target->opts->hfsp_block_size;
     cat_node_size = target->hfsp_cat_node_size;
 
     writer->compute_data_blocks = hfsplus_writer_compute_data_blocks;
