@@ -22,6 +22,97 @@
 #include <string.h>
 #include <stdio.h>
 
+
+int iso_imported_sa_new(struct iso_imported_sys_area **boots, int flag)
+{
+    struct iso_imported_sys_area *b;
+
+    *boots = NULL;
+    b = calloc(1, sizeof(struct iso_imported_sys_area));
+    if (b == NULL)
+        return ISO_OUT_OF_MEM;
+
+    b->mbr_req = NULL;
+    b->apm_req = NULL;
+
+    b->gpt_req = NULL;
+    b->gpt_backup_comments = NULL;
+
+    b->mips_boot_file_paths = NULL;
+    b->mips_vd_entries = NULL;
+
+    b->sparc_disc_label = NULL;
+    b->sparc_core_node = NULL;
+    b->sparc_entries = NULL;
+
+    b->hppa_cmdline = NULL;
+    b->hppa_bootloader = NULL;
+    b->hppa_kernel_32 = NULL;
+    b->hppa_kernel_64 = NULL;
+    b->hppa_ramdisk = NULL;
+
+    *boots = b;
+    return 1;
+}
+
+int iso_imported_sa_unref(struct iso_imported_sys_area **boots, int flag)
+{
+    int i;
+    struct iso_imported_sys_area *b;
+
+    b = *boots;
+    if (b == NULL)
+        return 2;
+    if (b->refcount > 0)
+        b->refcount--;
+    if (b->refcount > 0)
+        return 2;
+
+    if (b->mbr_req != NULL) {
+        for (i = 0; i < b->mbr_req_count; i++)
+            LIBISO_FREE_MEM(b->mbr_req[i]);
+        LIBISO_FREE_MEM(b->mbr_req);
+    }
+    if (b->apm_req != NULL) {
+        for (i = 0; i < b->apm_req_count; i++)
+            LIBISO_FREE_MEM(b->apm_req[i]);
+        LIBISO_FREE_MEM(b->apm_req);
+    }
+    if (b->gpt_req != NULL) {
+        for (i = 0; i < b->gpt_req_count; i++)
+            LIBISO_FREE_MEM(b->gpt_req[i]);
+        LIBISO_FREE_MEM(b->gpt_req);
+    }
+    LIBISO_FREE_MEM(b->gpt_backup_comments);
+
+    if (b->mips_boot_file_paths != NULL) {
+        for (i = 0; i < b->num_mips_boot_files; i++)
+            LIBISO_FREE_MEM(b->mips_boot_file_paths[i]);
+        LIBISO_FREE_MEM(b->mips_boot_file_paths);
+    }
+    if (b->mips_vd_entries != NULL) {
+        for (i = 0; i < b->num_mips_boot_files; i++)
+            LIBISO_FREE_MEM(b->mips_vd_entries[i]);
+        LIBISO_FREE_MEM(b->mips_vd_entries);
+    }
+    LIBISO_FREE_MEM(b->mipsel_boot_file_path);
+
+    LIBISO_FREE_MEM(b->sparc_disc_label);
+    if (b->sparc_core_node != NULL)
+        iso_node_unref((IsoNode *) b->sparc_core_node);
+    LIBISO_FREE_MEM(b->sparc_entries);
+
+    LIBISO_FREE_MEM(b->hppa_cmdline);
+    LIBISO_FREE_MEM(b->hppa_bootloader);
+    LIBISO_FREE_MEM(b->hppa_kernel_32);
+    LIBISO_FREE_MEM(b->hppa_kernel_64);
+    LIBISO_FREE_MEM(b->hppa_ramdisk);
+    LIBISO_FREE_MEM(b);
+    *boots = NULL;
+    return 1;
+}
+
+
 /**
  * Create a new image, empty.
  *
@@ -103,6 +194,7 @@ int iso_image_new(const char *name, IsoImage **image)
     for (i = 0; i < ISO_HFSPLUS_BLESS_MAX; i++)
         img->hfsplus_blessed[i] = NULL;
     img->collision_warnings = 0;
+    img->imported_sa_info = NULL;
 
     *image = img;
     return ISO_SUCCESS;
@@ -146,13 +238,7 @@ void iso_image_unref(IsoImage *image)
         iso_image_give_up_mips_boot(image, 0);
         if (image->sparc_core_node != NULL)
             iso_node_unref((IsoNode *) image->sparc_core_node);
-
-#ifdef Libisofs_enable_unreleased_hppa_palO
-
         iso_image_set_hppa_palo(image, NULL, NULL, NULL, NULL, NULL, 1);
-
-#endif
-
         free(image->volset_id);
         free(image->volume_id);
         free(image->publisher_id);
@@ -171,6 +257,7 @@ void iso_image_unref(IsoImage *image)
         if (image->system_area_data != NULL)
             free(image->system_area_data);
         iso_image_free_checksums(image, 0);
+        iso_imported_sa_unref(&(image->imported_sa_info), 0);
         free(image);
     }
 }
@@ -587,7 +674,8 @@ ex:;
 
 
 /**
- * A global counter for inode numbers for the ISO image filesystem.
+ * A global counter for Rock Ridge inode numbers in the ISO image filesystem.
+ *
  * On image import it gets maxed by the eventual inode numbers from PX
  * entries. Up to the first 32 bit rollover it simply increments the counter.
  * After the first rollover it uses a look ahead bitmap which gets filled
@@ -597,13 +685,13 @@ ex:;
  * @param image The image where the number shall be used
  * @param flag  bit0= reset count (Caution: image must get new inos then)
  * @return
- *     Since ino_t 0 is used as default and considered self-unique,
+ *     Since 0 is used as default and considered self-unique,
  *     the value 0 should only be returned in case of error.
  */
-ino_t img_give_ino_number(IsoImage *image, int flag)
+uint32_t img_give_ino_number(IsoImage *image, int flag)
 {
     int ret;
-    ino_t new_ino, ino_idx;
+    uint64_t new_ino, ino_idx;
     static uint64_t limit = 0xffffffff;
 
     if (flag & 1) {
@@ -613,10 +701,10 @@ ino_t img_give_ino_number(IsoImage *image, int flag)
         image->used_inodes = NULL;
         image->used_inodes_start = 0;
     }
-    new_ino = image->inode_counter + 1;
+    new_ino = ((uint64_t) image->inode_counter) + 1;
     if (image->used_inodes == NULL) {
         if (new_ino > 0 && new_ino <= limit) {
-            image->inode_counter = new_ino;
+            image->inode_counter = (uint32_t) new_ino;
             return image->inode_counter;
         }
     }
