@@ -241,8 +241,8 @@ ex:;
     return retval;
 }
 
-int strnconv(const char *str, const char *icharset, const char *ocharset,
-             size_t len, char **output)
+int strnconvl(const char *str, const char *icharset, const char *ocharset,
+              size_t len, char **output, size_t *out_len)
 {
     size_t inbytes;
     size_t outbytes;
@@ -278,7 +278,8 @@ int strnconv(const char *str, const char *icharset, const char *ocharset,
     *ret = '\0';
     iso_iconv_close(&conv, 0);
 
-    *output = malloc(ret - out + 1);
+    *out_len = ret - out;
+    *output = malloc(*out_len + 1);
     if (*output == NULL) {
         retval = ISO_OUT_OF_MEM;
         goto ex;
@@ -291,6 +292,15 @@ ex:;
     return retval;
 }
 
+int strnconv(const char *str, const char *icharset, const char *ocharset,
+             size_t len, char **output)
+{
+    size_t l;
+
+    return strnconvl(str, icharset, ocharset, len, output, &l);
+}
+
+
 /**
  * Convert a str in a specified codeset to WCHAR_T. 
  * The result must be free() when no more needed
@@ -667,6 +677,123 @@ int str2ucs(const char *icharset, const char *input, uint16_t **output)
     return ISO_SUCCESS;
 }
 
+int str2utf16be(const char *icharset, const char *input, uint16_t **output)
+{
+    int result;
+    wchar_t *wsrc_ = NULL;
+    char *src;
+    char *ret = NULL;
+    char *ret_ = NULL;
+    struct iso_iconv_handle conv;
+    int conv_ret = 0;
+    int direct_conv = 0;
+    size_t loop_counter = 0, loop_limit = 3;
+    size_t numchars;
+    size_t outbytes;
+    size_t inbytes;
+    size_t n;
+
+    if (icharset == NULL || input == NULL || output == NULL) {
+        return ISO_NULL_POINTER;
+    }
+
+    /* 
+      Try the direct conversion.
+    */ 
+    conv_ret = iso_iconv_open(&conv, "UTF-16BE", (char *) icharset, 0);
+    if (conv_ret > 0) {
+        direct_conv = 1;
+        src = (char *) input;
+        inbytes = strlen(input);
+        loop_limit = inbytes + 3;
+        outbytes = (2 * inbytes + 1) * sizeof(uint16_t);
+        ret_ = malloc(outbytes);
+        if (ret_ == NULL)
+            return ISO_OUT_OF_MEM;
+        ret = ret_;
+    }  else {
+        /* Try via intermediate character set WCHAR_T.
+        */
+        result = str2wchar(icharset, input, &wsrc_);
+        if (result == (int) ISO_SUCCESS) {
+            src = (char *)wsrc_;
+            numchars = wcslen(wsrc_);
+
+            inbytes = numchars * sizeof(wchar_t);
+            loop_limit = inbytes + 3;
+
+            ret_ = malloc((2 * numchars+1) * sizeof(uint16_t));
+            if (ret_ == NULL)
+                return ISO_OUT_OF_MEM;
+            outbytes = 2 * numchars * sizeof(uint16_t);
+            ret = ret_;
+
+            /* initialize iconv */
+            conv_ret = iso_iconv_open(&conv, "UTF-16BE", "WCHAR_T", 0);
+            if (conv_ret <= 0) {
+                free(wsrc_);
+                free(ret_);
+            }
+        } else if (result != (int) ISO_CHARSET_CONV_ERROR)
+            return result;
+    }
+
+    if (conv_ret <= 0) {
+        return ISO_CHARSET_CONV_ERROR;
+    }
+
+    n = iso_iconv(&conv, &src, &inbytes, &ret, &outbytes, 0);
+    while (n == (size_t) -1) {
+        /* The destination buffer is too small. Stops here. */
+        if (errno == E2BIG)
+            break;
+
+        /* An incomplete multi bytes sequence was found. We 
+         * can't do anything here. That's quite unlikely. */
+        if (errno == EINVAL)
+            break;
+
+        /* The last possible error is an invalid multi bytes
+         * sequence. Just replace the character with a "_". 
+         * Probably the character doesn't exist in UCS */
+        set_ucsbe((uint16_t*) ret, '_');
+        ret += sizeof(uint16_t);
+        outbytes -= sizeof(uint16_t);
+
+        if (!outbytes)
+            break;
+
+        /* There was an error with one character but some other remain
+         * to be converted. That's probably a multibyte character.
+         * See above comment. */
+        if (direct_conv) {
+            src++;
+            inbytes--;
+        } else {
+            src += sizeof(wchar_t);
+            inbytes -= sizeof(wchar_t);
+        }
+
+        if (!inbytes)
+            break;
+
+        /* Just to appease my remorse about unclear loop ends */
+        loop_counter++;
+        if (loop_counter > loop_limit)
+            break;
+        n = iso_iconv(&conv, &src, &inbytes, &ret, &outbytes, 0);
+    }
+    iso_iconv_close(&conv, 0);
+
+    /* close the UTF-16 string */
+    set_ucsbe((uint16_t*) ret, '\0');
+    if (wsrc_ != NULL)
+        free(wsrc_);
+
+    *output = (uint16_t*)ret_;
+    return ISO_SUCCESS;
+}
+
 static int valid_d_char(char c)
 {
     return (c >= '0' && c <= '9') || (c >= 'A' && c <= 'Z') || (c == '_');
@@ -1080,7 +1207,7 @@ uint16_t *iso_j_file_id(const uint16_t *src, int flag)
 {
     uint16_t *dot, *retval = NULL;
     size_t lname, lext, lnname, lnext, pos, i, maxchar = 64;
-    uint16_t *dest = NULL;
+    uint16_t *dest = NULL, c;
 
     LIBISO_ALLOC_MEM_VOID(dest, uint16_t, LIBISO_JOLIET_NAME_MAX);
                                /* was: 66 = 64 (name + ext) + 1 (.) + 1 (\0) */
@@ -1120,7 +1247,7 @@ uint16_t *iso_j_file_id(const uint16_t *src, int flag)
 
     /* Convert up to lnname characters of the filename. */
     for (i = 0; i < lnname; i++) {
-        uint16_t c = src[i];
+        c = src[i];
         if (valid_j_char(c)) {
             dest[pos++] = c;
         } else {
@@ -1128,6 +1255,8 @@ uint16_t *iso_j_file_id(const uint16_t *src, int flag)
             pos++;
         }
     }
+    if (pos > 0)
+        iso_handle_split_utf16(dest + (pos - 1));
 
     if ((flag & 1) && lnext <= 0)
         goto is_done;
@@ -1145,6 +1274,7 @@ uint16_t *iso_j_file_id(const uint16_t *src, int flag)
             pos++;
         }
     }
+    iso_handle_split_utf16(dest + (pos - 1));
 
 is_done:;
     set_ucsbe(dest + pos, '\0');
@@ -1181,6 +1311,7 @@ uint16_t *iso_j_dir_id(const uint16_t *src, int flag)
             set_ucsbe(dest + i, '_');
         }
     }
+    iso_handle_split_utf16(dest + (len - 1));
     set_ucsbe(dest + len, '\0');
     retval = ucsdup(dest);
 ex:
@@ -1225,12 +1356,13 @@ uint16_t *ucsdup(const uint16_t *str)
 
 /**
  * Although each character is 2 bytes, we actually compare byte-by-byte
- * (thats what the spec says).
+ * because the words are big-endian. Comparing possibly swapped words
+ * would make the sorting order depend on the machine byte order.
  */
 int ucscmp(const uint16_t *s1, const uint16_t *s2)
 {
-    const char *s = (const char*)s1;
-    const char *t = (const char*)s2;
+    const uint8_t *s = (const uint8_t*)s1;
+    const uint8_t *t = (const uint8_t*)s2;
     size_t len1 = ucslen(s1);
     size_t len2 = ucslen(s2);
     size_t i, len = MIN(len1, len2) * 2;
@@ -1261,6 +1393,8 @@ uint16_t *ucsncpy(uint16_t *dest, const uint16_t *src, size_t n)
 {
     n = MIN(n, ucslen(src) + 1);
     memcpy(dest, src, n*2);
+    if (n >= 2)
+        iso_handle_split_utf16(dest + (n - 2));
     return dest;
 }
 
@@ -1354,6 +1488,18 @@ void iso_bb(uint8_t *buf, uint32_t num, int bytes)
     iso_msb(buf+bytes, num, bytes);
 }
 
+/* An alternative to iso_lsb() which advances the write pointer
+*/
+int iso_lsb_to_buf(char **wpt, uint32_t value, int bytes, int flag)
+{
+    int b, bits;
+
+    bits = bytes * 8;
+    for (b = 0; b < bits; b += 8)
+        *((unsigned char *) ((*wpt)++)) = (value >> b) & 0xff;
+    return (1);
+}
+
 uint32_t iso_read_lsb(const uint8_t *buf, int bytes)
 {
     int i;
@@ -1388,6 +1534,26 @@ uint32_t iso_read_bb(const uint8_t *buf, int bytes, int *error)
     return v1;
 }
 
+uint64_t iso_read_lsb64(const uint8_t *buf)
+{
+    int i;
+    uint64_t ret = 0;
+
+    for (i=0; i < 8; i++)
+        ret += ((uint64_t) buf[i]) << (i * 8);
+    return ret;
+}
+
+uint64_t iso_read_msb64(const uint8_t *buf)
+{
+    int i;
+    uint64_t ret = 0;
+
+    for (i=0; i < 8; i++)
+        ret += ((uint64_t) buf[7 - i]) << (i * 8);
+    return ret;
+}
+
 void iso_datetime_7(unsigned char *buf, time_t t, int always_gmt)
 {
     static int tzsetup = 0;
@@ -1865,6 +2031,17 @@ int iso_util_dec_to_uint32(char *dec, uint32_t *value, int flag)
 }
 
 
+int iso_util_bin_to_hex(char *target, uint8_t *bytes, int num_bytes, int flag)
+{
+    int i;
+
+    for (i = 0; i < num_bytes; i++)
+        sprintf(target + 2 * i, "%-2.2x", bytes[i]);
+    target[2 * num_bytes] = 0;
+    return 1;
+}
+
+
 int iso_util_hex_to_bin(char *hex, char *bin, int bin_size, int *bin_count,
                         int flag)
 {
@@ -2063,4 +2240,56 @@ void *iso_alloc_mem(size_t size, size_t count, int flag)
 	iso_msg_submit(-1, ISO_OUT_OF_MEM, 0, "Out of virtual memory");
     return pt;
 }
- 
+
+
+uint16_t iso_ntohs(uint16_t v)
+{
+    return iso_read_msb((uint8_t *) &v, 2);
+}
+
+uint16_t iso_htons(uint16_t v)
+{
+    uint16_t ret;
+
+    iso_msb((uint8_t *) &ret, (uint32_t) v, 2);
+
+    return ret;
+}   
+
+
+/* If an UTF-16 surrogate pair was split : Change to UTF-16 '_'.
+   (UCS-2 is promised to reserve 0xd800 to 0xdbff for UTF-16).
+*/
+void iso_handle_split_utf16(uint16_t *utf_word)
+{
+    unsigned char *hb;
+
+    hb = (unsigned char *) utf_word;
+    if ((hb[0] & 0xfc) == 0xd8)
+        set_ucsbe(utf_word, '_');
+}
+
+
+int iso_clone_mem(char *in, char **out, size_t size)
+{
+    if (in == NULL) {
+        *out = NULL;
+        return 1;
+    }
+    if (size == 0)
+        size = strlen(in) + 1;
+    *out = calloc(1, size);
+    if (*out == NULL)
+        return ISO_OUT_OF_MEM;
+    memcpy(*out, in, size);
+    return 1;
+}
+    
+
+int iso_clone_mgtd_mem(char *in, char **out, size_t size)
+{
+    if (*out != NULL)
+        free(*out);
+    return iso_clone_mem(in, out, size);
+}
+
