@@ -56,9 +56,16 @@ int iso_init_locale(int flag);
 int strconv(const char *input, const char *icharset, const char *ocharset,
             char **output);
 
+/* Like strconv but processing len input bytes rather than strlen(input)
+ */
 int strnconv(const char *str, const char *icharset, const char *ocharset,
              size_t len, char **output);
 
+/* Like strnconv but also returning the number of bytes in *output.
+ */
+int strnconvl(const char *str, const char *icharset, const char *ocharset,
+              size_t len, char **output, size_t *out_len);
+
 /**
  * Convert a given string from any input charset to ASCII
  * 
@@ -88,6 +95,22 @@ int str2ascii(const char *icharset, const char *input, char **output);
  */
 int str2ucs(const char *icharset, const char *input, uint16_t **output);
 
+/**
+ * Convert a given string from any input charset to UTF-16BE charset,
+ * used for HFS+ file identifiers.
+ * (UTF-16 differs from older UCS-2 by having multi word characters.)
+ * 
+ * @param icharset
+ *      Input charset. Must be supported by iconv
+ * @param input
+ *      Input string
+ * @param output
+ *      Location where the pointer to the ouput string will be stored
+ * @return
+ *      1 on success, < 0 on error
+ */
+int str2utf16be(const char *icharset, const char *input, uint16_t **output);
+
 /**
  * Create a level 1 directory identifier.
  * 
@@ -220,6 +243,12 @@ uint16_t *ucscpy(uint16_t *dest, const uint16_t *src);
  */
 uint16_t *ucsncpy(uint16_t *dest, const uint16_t *src, size_t n);
 
+/**
+ * Check whether utf_word is the first surrogate word of a pair.
+ * If so, change it to UTF-16 character '_'.
+ */
+void iso_handle_split_utf16(uint16_t *utf_word);
+
 /**
  * Convert a given input string to d-chars.
  * @return
@@ -244,6 +273,9 @@ uint32_t iso_read_msb(const uint8_t *buf, int bytes);
  */
 uint32_t iso_read_bb(const uint8_t *buf, int bytes, int *error);
 
+uint64_t iso_read_lsb64(const uint8_t *buf);
+uint64_t iso_read_msb64(const uint8_t *buf);
+
 /** 
  * Records the date/time into a 7 byte buffer (ECMA-119, 9.1.5)
  * 
@@ -515,6 +547,12 @@ int iso_util_eval_md5_tag(char *block, int desired, uint32_t lba,
 int iso_util_tag_magic(int tag_type, char **tag_magic, int *len, int flag);
 
 
+int iso_util_bin_to_hex(char *target, uint8_t *bytes, int num_bytes, int flag);
+
+int iso_util_hex_to_bin(char *hex, char *bin, int bin_size, int *bin_count,
+                        int flag);
+
+
 /* ------------------------------------------------------------------------- */
 
 /* In md5.h these function prototypes would be neighbors of (Ecma119Image *)
@@ -577,6 +615,19 @@ void *iso_alloc_mem(size_t size, size_t count, int flag);
         }
 
 
+/*
+   @param in    Valid memory or NULL
+   @param out   Returns valid memory or NULL
+   @param size  Number of bytes to copy. 0 means strlen(in)+1 if not NULL.
+   @return      1 or ISO_OUT_OF_MEM
+*/
+int iso_clone_mem(char *in, char **out, size_t size);
+
+/* Like iso_clone_mem but first freeing *out if not NULL
+*/
+int iso_clone_mgtd_mem(char *in, char **out, size_t size);
+
+
 /* ------------------------------------------------------------------------- */
 
 
