@@ -3,7 +3,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -111,6 +111,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
    free(leafname);
    return(-1);
  }
+
+ /* Base initialization by actions which must not fail */
+
  m->libs_are_started= 0;
  strncpy(m->progname,progname,sizeof(m->progname)-1);
  m->progname[sizeof(m->progname)-1]= 0;
@@ -157,6 +160,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_follow_param= 0;
  m->do_follow_links= 0;
  m->follow_link_limit= 100;
+ m->resolve_link_rec_count= 0;
+ m->resolve_link_rec_limit= 100;
+ m->do_follow_concat= 0;
  m->do_follow_mount= 1;
  m->do_global_uid= 0;
  m->global_uid= 0;
@@ -237,6 +243,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_stream_recording= 0;
  m->dvd_obs= 0;
  m->stdio_sync= 0;
+ m->stdio_sync_is_default= 1;
  m->keep_boot_image= 0;
  m->boot_image_cat_path[0]= 0;
  m->boot_image_cat_hidden= 0;
@@ -427,6 +434,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
 
  m->info_text[0]= 0;
 
+
+ /* Here begin actions which might fail */
+
  ret= Sfile_leafname(progname, leafname, 0);
  if(ret<=0)
    goto failure;
