@@ -35,9 +35,10 @@
 
 
 /* @param flag bit0= simple readlink(): no normalization, no multi-hop
+               bit1= this is potentially a recursion
 */
-int Xorriso_resolve_link(struct XorrisO *xorriso,
-                     char *link_path, char result_path[SfileadrL], int flag)
+int Xorriso_resolve_link(struct XorrisO *xorriso, char *link_path,
+                         char result_path[SfileadrL], int flag)
 {
  ssize_t l;
  struct stat stbuf;
@@ -45,6 +46,19 @@ int Xorriso_resolve_link(struct XorrisO *xorriso,
  char *buf= NULL, *dirbuf= NULL, *lpt, *spt;
  static int link_limit= 100;
 
+ if(flag & 1) {
+   xorriso->resolve_link_rec_count++;
+   if(xorriso->resolve_link_rec_count > xorriso->resolve_link_rec_limit) {
+     Xorriso_msgs_submit(xorriso, 0, link_path, 0, "ERRFILE", 0);
+     sprintf(xorriso->info_text, "Too many link recursions before : ");
+     Text_shellsafe(link_path, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, show_errno,
+                         "FAILURE", 0);
+     {ret= 0; goto ex;}
+   }
+ } else
+   xorriso->resolve_link_rec_count= 0;
+
  Xorriso_alloc_meM(buf, char, SfileadrL);
  Xorriso_alloc_meM(dirbuf, char, SfileadrL);
 
@@ -99,8 +113,9 @@ handle_abort:;
    break;
    }
    ret= Xorriso_normalize_img_path(xorriso, dirbuf, buf, result_path, 2|4);
-   if(ret<=0)
+   if(ret<=0) {
      goto ex;
+   }
 
    if(lstat(result_path, &stbuf)==-1) {
      lpt= result_path;
@@ -125,6 +140,8 @@ too_many_hops:;
 ex:;
  Xorriso_free_meM(buf);
  Xorriso_free_meM(dirbuf);
+ if(xorriso->resolve_link_rec_count > 0)
+   xorriso->resolve_link_rec_count--;
  return(ret);
 }
 
@@ -544,9 +561,12 @@ revoke_sorting:;
    report_size= sub_size/1024;
    if(report_size*1024<sub_size)
       report_size++;
-   sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
+   if(xorriso->sh_style_result)
+     sprintf(xorriso->result_line, "%-7.f ",(double) (report_size));
+   else
+     sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
    sprintf(xorriso->result_line+strlen(xorriso->result_line), "%s\n",
-           Text_shellsafe(show_path, sfe, 0));
+           Xorriso_esc_filepath(xorriso,show_path, sfe, 0));
    Xorriso_result(xorriso, 0);
  }
 
@@ -803,7 +823,7 @@ int Xorriso_lsx_filev(struct XorrisO *xorriso, char *wd,
 
  /* Count valid nodes, warn of invalid ones */
  for(i= 0; i<filec; i++) {
-   ret= Xorriso_make_abs_adr(xorriso, wd, filev[i], path, 1|2|4);
+   ret= Xorriso_make_abs_adr(xorriso, wd, filev[i], path, 1|2|4|8);
    if(ret<=0) {
      was_error++;
  continue;
@@ -848,7 +868,7 @@ int Xorriso_lsx_filev(struct XorrisO *xorriso, char *wd,
      if(filec>1) {
        strcpy(xorriso->result_line, "\n");
        Xorriso_result(xorriso,0);
-       Text_shellsafe(filev[i], xorriso->result_line, 0);
+       Xorriso_esc_filepath(xorriso,filev[i], xorriso->result_line, 0);
        strcat(xorriso->result_line, ":\n");
        Xorriso_result(xorriso,0);
      }
@@ -894,14 +914,17 @@ int Xorriso_lsx_filev(struct XorrisO *xorriso, char *wd,
        if(ret==0)
  continue;
      }
-     sprintf(rpt, "%7.f ",(double) (size/1024));
+     if(xorriso->sh_style_result)
+       sprintf(rpt, "%-7.f ",(double) (size/1024));
+     else
+       sprintf(rpt, "%7.f ",(double) (size/1024));
    }
    if(link_target[0]) {
-     Text_shellsafe(filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso,filev[i], xorriso->result_line, 1);
      strcat(xorriso->result_line, " -> ");
-     Text_shellsafe(link_target, xorriso->result_line, 1 | 2);
+     Xorriso_esc_filepath(xorriso,link_target, xorriso->result_line, 1 | 2);
    } else {
-     Text_shellsafe(filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso,filev[i], xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1225,7 +1248,7 @@ int Xorriso_findx_action(struct XorrisO *xorriso, struct FindjoB *job,
  } else if(action == 44) { /* list_extattr */
    ret= Xorriso_list_extattr(xorriso, NULL, abs_path, show_path, target, 2);
  } else {
-   Text_shellsafe(show_path, xorriso->result_line, 0);
+   Xorriso_esc_filepath(xorriso,show_path, xorriso->result_line, 0);
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
    ret= 1;
@@ -1483,19 +1506,30 @@ ex:;
 
 /* @param flag bit0= path is a directory
                bit2= recursion: do not reassure in mode 2 "tree"
-               bit3= this is for overwriting and not for plain removal
+             bit3-7= question text mode
+                     0= plain removal 
+                     1= replacing file object
+                     2= overwriting of content, keeping file object
+                     3= appending of content, keeping file object
 */
 int Xorriso_reassure_restore(struct XorrisO *xorriso, char *path, int flag)
 {
- int ret;
+ int ret, mode;
 
+ mode= (flag >> 3) & 31;
  while((xorriso->do_reassure==1 || (xorriso->do_reassure==2 && !(flag&4)))
        && !xorriso->request_not_to_ask) {
    /* ls -ld */
    Xorriso_lsx_filev(xorriso, xorriso->wdx, 1, &path, (off_t) 0, 1|2|8);
    if(flag&1) /* du -s */
      Xorriso_lsx_filev(xorriso, xorriso->wdx, 1, &path, (off_t) 0, 2|4);
-   if(flag&8)
+   if(mode == 3)
+     sprintf(xorriso->info_text,
+   "File exists. Append content ?  n= no, y= yes, x= abort, @= stop asking\n");
+   else if(mode == 2)
+     sprintf(xorriso->info_text,
+"File exists. Overwrite content ?  n= no, y= yes, x= abort, @= stop asking\n");
+   else if(mode == 1)
      sprintf(xorriso->info_text,
   "File exists. Remove ?  n= keep old, y= remove, x= abort, @= stop asking\n");
    else
@@ -1507,7 +1541,7 @@ int Xorriso_reassure_restore(struct XorrisO *xorriso, char *path, int flag)
      goto ex;
    if(xorriso->request_to_abort) {
      sprintf(xorriso->info_text,
-             "Removal operation aborted by user before file: ");
+             "File alteration operation aborted by user before file: ");
      Text_shellsafe(path, xorriso->info_text, 1);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
      ret= 3; goto ex;
@@ -1775,7 +1809,7 @@ int Xorriso_afile_fopen(struct XorrisO *xorriso,
  if(fp==NULL){
    if(!(flag&1)) {
      sprintf(xorriso->info_text,
-             "Failed to open file '%s' in %s mode\n", filename, mode);
+             "Failed to open file '%s' in %s mode", filename, mode);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE", 0);
    }
    return(0);
@@ -1789,6 +1823,7 @@ int Xorriso_afile_fopen(struct XorrisO *xorriso,
    @param flag bit0= make absolute command paths with known systems
                bit1= do not allow prefixes with cmd
                bit2= interpret unprefixed cmd as shell:
+               bit3= do not care for device filetype
 */
 int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
                            int lba, int track, int session, char *volid,
@@ -1797,18 +1832,21 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
  int ret, reg_file= 0, is_safe= 0, sys_code= 0;
  char *form= NULL, session_text[12], track_text[12], lba_text[12];
  char *vars[5][2], *sfe= NULL, *volid_sfe= NULL, *cpt, *sysname;
+ char *cooked_dev= NULL, *eff_dev;
  struct stat stbuf;
 
  Xorriso_alloc_meM(form, char, 6 * SfileadrL);
  Xorriso_alloc_meM(sfe, char, 5 * SfileadrL);
  Xorriso_alloc_meM(volid_sfe, char, 5 * 80 + 1);
+ Xorriso_alloc_meM(cooked_dev, char, SfileadrL);
 
  if(strlen(cmd) > SfileadrL) {
    Xorriso_msgs_submit(xorriso, 0, "Argument much too long", 0, "FAILURE", 0);
    {ret= 0; goto ex;}
  }
+ eff_dev= devadr;
  ret= stat(devadr, &stbuf);
- if(ret != -1)
+ if(ret != -1 && !(flag & 8))
    if(S_ISREG(stbuf.st_mode))
      reg_file= 1;
  if(strncmp(cmd, "linux:", 6) == 0 && !(flag & 2)) {
@@ -1817,6 +1855,9 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
  } else if(strncmp(cmd, "freebsd:", 8) == 0 && !(flag & 2)) {
    cpt= cmd + 8;
    sys_code= 2;
+ } else if(strncmp(cmd, "netbsd:", 7) == 0 && !(flag & 2)) {
+   cpt= cmd + 7;
+   sys_code= 3;
  } else if(strncmp(cmd, "string:", 7) == 0 && !(flag & 2)) {
    cpt= cmd + 7;
    strcpy(form, cpt);
@@ -1835,6 +1876,8 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
              strcmp(sysname, "GNU/kFreeBSD") == 0) {
                                          /* "GNU/kFreeBSD" = Debian kfreebsd */
      sys_code= 2;
+   } else if(strcmp(sysname, "NetBSD") == 0) {
+     sys_code= 3;
    } else if(strcmp(sysname, "Linux") == 0) {
      sys_code= 1;
    } else {
@@ -1852,7 +1895,7 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
            (reg_file || (xorriso->mount_opts_flag & 1) ? "loop," : ""),
            Text_shellsafe(cpt, sfe, 0));
    is_safe= 1;
- } else if(sys_code == 2) { /* FreeBSD */
+ } else if(sys_code == 2 || sys_code == 3) { /* FreeBSD , NetBSD */
    if(reg_file) {
 
 /*  <<< Considered to create vnode as of
@@ -1866,20 +1909,26 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
 */
 
      Xorriso_msgs_submit(xorriso, 0, 
-           "Detected regular file as mount device with FreeBSD style command.",
+           "Detected regular file as mount device with BSD style command.",
            0, "FAILURE", 0);
-     Xorriso_msgs_submit(xorriso, 0,
+     if(sys_code == 2) {
+       Xorriso_msgs_submit(xorriso, 0,
 "Command mdconfig -a -t vnode -f can create a device node which uses the file",
                          0, "HINT", 0);
+     } else {
+       Xorriso_msgs_submit(xorriso, 0,
+     "Command  vnconfig -c vndX  can create a device node which uses the file",
+                         0, "HINT", 0);
+     }
      {ret= 0; goto ex;}
    } else {
+     if(sys_code == 3 && strncmp(devadr, "/dev/rcd", 8) == 0) {
+       sprintf(cooked_dev, "/dev/cd%s", devadr + 8);
+       eff_dev= cooked_dev;
+     }
      sprintf(form,
          "%smount_cd9660 -o noexec,nosuid -s %%sbsector%% %%device%% %s",
-         (flag & 1 ? "/sbin/" : ""), Text_shellsafe(cmd+8, sfe, 0));
-     /*
-       Not working on FreeBSD 7.2 according to Zsolt Kuti, 11 Oct 2009:
-         "%smount -t cd9660 -o noexec,nosuid -o -s %%sbsector%% %%device%% %s",
-     */
+         (flag & 1 ? "/sbin/" : ""), Text_shellsafe(cpt, sfe, 0));
    }
    is_safe= 1;
  }
@@ -1895,15 +1944,16 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
  vars[3][0]= "volid";
  vars[3][1]= Text_shellsafe(volid, volid_sfe, 0);
  vars[4][0]= "device";
- vars[4][1]= Text_shellsafe(devadr, sfe, 0);
+ vars[4][1]= Text_shellsafe(eff_dev, sfe, 0);
  ret= Sregex_resolve_var(form, vars, 5, "%", "%", "%", result, SfileadrL, 0);
  if(ret <= 0)
    goto ex;
  ret= 1 + is_safe;
 ex:;
- Xorriso_free_meM(form);
- Xorriso_free_meM(sfe);
+ Xorriso_free_meM(cooked_dev);
  Xorriso_free_meM(volid_sfe);
+ Xorriso_free_meM(sfe);
+ Xorriso_free_meM(form);
  return(ret);
 }
 
@@ -1937,3 +1987,185 @@ int Xorriso_append_scdbackup_record(struct XorrisO *xorriso, int flag)
  return(1);
 }
 
+
+/* @param flag bit0= for append rather than overwrite
+*/
+int Xorriso_is_concat_target(struct XorrisO *xorriso,  char *target,
+                             int *ftype, int *fd, int flag)
+{
+ int ret;
+ char *why= "";
+
+ *ftype= 0;
+ *fd = -1;
+
+ if(strcmp(target, "-") == 0) {
+   *fd= 1;
+   *ftype= 8; /* character device */
+   return(1);
+ }
+
+ *ftype= Sfile_type(target, 1 | 8 | 16);
+ if(*ftype == -1)
+   return(2); /* not yet existing regular */
+
+ if(*ftype & 1024) {
+   *fd= Sfile_get_dev_fd_no(target, 0);
+   *ftype &= ~1024;
+ }
+ if(*ftype & 2048) {
+   why= "fstat(2) returned -1 on file descriptor number.";
+   goto not_usable;
+ }
+
+ if(*ftype == 3) {
+   if(!xorriso->do_follow_concat) {
+     why= "May not follow symbolic link. No -follow \"...:concat:...\".";
+     goto not_usable;
+   }
+   *ftype= Sfile_type(target, 1 | 4 | 8);
+   if(*ftype == -1)
+     return(2); /* not yet existing regular */
+ }
+ if(*ftype == 2) {
+   why= "May not write data into a directory.";
+   goto not_usable;
+ }
+ if(*ftype == 0) {
+   why= "Cannot determine file type.";
+   goto not_usable;
+ }
+ if(*ftype == 7) {
+
+   /* >>> what to do with UNIX socket ? */;
+
+   why= "Cannot yet handle socket file as target.";
+   goto not_usable;
+ }
+
+ if(xorriso->do_overwrite != 1 && xorriso->do_overwrite != 2) {
+   why= "May not alter existing file.";
+   goto not_usable;
+ }
+ ret= Xorriso_reassure_restore(xorriso, target, (2 + (flag & 1)) << 3);
+ if(ret <= 0 || ret == 0) {
+   why= "User revoked alteration of existing file.";
+   goto not_usable;
+ }
+
+ if(*ftype == 1)
+   return(2); /* existing regular */
+ if(*ftype == 4 || *ftype == 6 || *ftype == 8)
+   return(1); /* named pipe, block device, character device */
+
+not_usable:; 
+ sprintf(xorriso->info_text,
+               "Unsuitable -concat target: ");
+ Text_shellsafe(target, xorriso->info_text, 1);
+ sprintf(xorriso->info_text + strlen(xorriso->info_text), ". %s", why);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ return(0);
+}
+
+
+int Xorriso_concat(struct XorrisO *xorriso, char *mode, char *target,
+                   int progc, char **progv,
+                   int filec, char **filev, int flag)
+{
+ int ret, i, fd= -1, target_is_regular= 0, fd_opened= 0, ftype, prog_forked= 0;
+ int status;
+ int open_mode= O_WRONLY | O_CREAT;
+ struct stat stbuf;
+ pid_t forked_pid;
+
+ for(i= 0; i < filec; i++) {
+   ret= Xorriso_iso_lstat(xorriso, filev[i], &stbuf, 4);
+   if(ret == -1)
+     goto ex;
+   if(!S_ISREG(stbuf.st_mode)) {
+     sprintf(xorriso->info_text,
+             "-concat: iso_rr_path is not a regular data file: ");
+     Text_shellsafe(filev[i], xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+ }
+ if(strcmp(mode, "overwrite") == 0) {
+   ret= Xorriso_is_concat_target(xorriso, target, &ftype, &fd, 0);
+   if(ret <= 0)
+     goto ex;
+   if(ret == 2) {
+     target_is_regular= 1;
+     open_mode |= O_TRUNC;
+   }
+   if(fd == -1) {
+     fd= open(target, O_WRONLY | O_TRUNC | O_CREAT, 0666);
+     fd_opened= 1;
+   }
+ } else if(strcmp(mode, "append") == 0) {
+   ret= Xorriso_is_concat_target(xorriso, target, &ftype, &fd, 1);
+   if(ret <= 0)
+     goto ex;
+   target_is_regular= (ret == 2);
+   if(fd == -1) {
+     fd= open(target, O_WRONLY | O_CREAT, 0666);
+     fd_opened= 1;
+     if(fd != -1 && target_is_regular) {
+       ret= lseek(fd, (off_t) 0, SEEK_END);
+       if(ret == -1) {
+         sprintf(xorriso->info_text,
+                 "-concat append: Cannot lseek(2) to file end of ");
+         Text_shellsafe(target, xorriso->info_text, 1);
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+         ret= 0; goto ex;
+       }
+     }
+   }
+
+ } else if(strcmp(mode, "pipe") == 0) {
+   ret= Xorriso_pipe_open(xorriso, "-concat pipe", progv[0], progc, progv,
+                          "", &fd, &forked_pid, 2 | 8);
+   if(ret <= 0)
+     goto ex;
+   fd_opened= 1;
+   prog_forked= 1;
+
+ } else {
+   sprintf(xorriso->info_text, "-concat: Unknown mode ");
+   Text_shellsafe(mode, xorriso->info_text, 1);
+   strcat(xorriso->info_text, ". Known modes: overwrite, append, pipe");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if(fd == -1) {
+   sprintf(xorriso->info_text,
+           "-concat: Cannot open file handle to ");
+   Text_shellsafe(target, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+
+ for(i= 0; i < filec; i++) {
+   ret= Xorriso_iso_file_to_fd(xorriso, filev[i], fd, 0);
+   if(ret <= 0) {
+     if(i < filec - 1) {
+       ret= Xorriso_eval_problem_status(xorriso, ret, 1 | 2);
+       if(ret < 0) {
+         sprintf(xorriso->info_text,
+                 "-concat: Aborted although %d files stay unprocessed.",
+                 filec - i + 1);
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+         ret= 0; goto ex;
+       }
+     }
+   }
+ }
+
+ ret= 1; 
+ex:;
+ if(fd_opened && fd != -1)
+   close(fd);
+ if(prog_forked)
+   Xorriso_wait_child_end(xorriso, forked_pid, &status, 0);
+ return(ret);
+}
