@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -82,7 +82,7 @@ int Xorriso_cdrskin_help(struct XorrisO *xorriso, int flag)
 "\twrite_start_address=#\t\twrite to byte address on DVD+RW, BD-RE",
 "\tstream_recording=on|number\ttry to get full speed on DVD-RAM, BD",
 "\tdvd_obs=default|32k|64k\t\tbytes per DVD/BD write operation",
-"\tstdio_sync=on|off|number\twhether to fsync output to \"stdio:\"",
+"\tstdio_sync=on|off|end|number\twhether to fsync output to \"stdio:\"",
 "\t--no_rc\t\tDo not execute xorriso startup files",
 "\t-help\t\tprint this text to stderr and exit emulation",
 "Actually this is the integrated ISO RockRidge filesystem manipulator xorriso",
@@ -597,7 +597,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "--old-root-no-md5", "--old-root-devno", "--old-root-no-ino",
     "--no_rc", "--norock", "-hfsplus", "-fat", "-chrp-boot-part",
     "-isohybrid-gpt-basdat", "-isohybrid-gpt-hfsplus",
-    "-isohybrid-apm-hfsplus", "--grub2-boot-info",
+    "-isohybrid-apm-hfsplus", "--grub2-boot-info", "-joliet-utf16",
     ""
  };
  static char arg1_options[][41]= {
@@ -635,6 +635,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-rr_reloc_dir", "-hfsplus-serial-no", "-prep-boot-part", "-efi-boot-part",
     "-hfsplus-block-size", "-apm-block-size", "--grub2-mbr",
     "--grub2-sparc-core", "--sort-weight-list", "--sort-weight-patterns",
+    "-hppa-hdrversion",
     ""
  };
  static char arg2_options[][41]= {
@@ -761,6 +762,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -disallow_dir_id_ext        Do not allow dot in ISO directory names",
 "  -J, -joliet                 Generate Joliet directory information",
 "  -joliet-long                Allow Joliet file names to be 103 Unicode characters",
+"  -joliet-utf16               Encode Joliet names in UTF-16BE rather than UCS-2",
 "  -U, -untranslated-filenames Allow Untranslated filenames (for HPUX & AIX - violates ISO9660).",
 "  -untranslated_name_len LEN  Allow up to LEN (1..96) name characters (heavily violates ISO9660).",
 "  -allow-lowercase            Allow lower case characters in addition to the current character set (violates ISO9660)",
@@ -852,6 +854,12 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -mipsel-boot FILE           Set mipsel boot image name (relative to image root)",
 "  -B FILES, -sparc-boot FILES Set sparc boot image names",
 "  -sparc-label label text     Set sparc boot disk label",
+"  -hppa-cmdline CMDLINE       Set hppa boot command line",
+"  -hppa-kernel-32 FILE        Set hppa 32-bit image name (relative to image root)",
+"  -hppa-kernel-64 FILE        Set hppa 64-bit image name (relative to image root)",
+"  -hppa-bootloader FILE       Set hppa boot loader file name (relative to image root)",
+"  -hppa-ramdisk FILE          Set hppa ramdisk file name (relative to image root)",
+"  -hppa-hdrversion NUMBER     Set hppa PALO header version to 4 or 5",
 "  --grub2-sparc-core FILE     Set path of core file for disk label patching",
 "  -efi-boot-part DISKFILE|--efi-boot-image",
 "                              Set data source for EFI System Partition",
@@ -1142,6 +1150,10 @@ int Xorriso_genisofs_path_pecul(struct XorrisO *xorriso, int *was_path,
  /* mkisofs records mtime in ECMA-119 and Joliet
  */
  Xorriso_relax_compliance(xorriso, "rec_mtime", 0);
+ /* mkisofs is substantially faster than xorriso by not fsyncing
+ */
+ if(xorriso->stdio_sync_is_default)
+   Xorriso_option_stdio_sync(xorriso, "off", 0);
 
  Xorriso_free_meM(sfe);
  ret= 1;
@@ -1155,7 +1167,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
                      int argc, char **argv, int flag)
 {
  int ret, i, j, was_path= 0, was_other_option= 0, mem_graft_points, mem;
- int do_print_size= 0, fd, idx, iso_level= -1;
+ int do_print_size= 0, fd, idx, iso_level= -1, palohdrversion;
  int was_failure= 0, fret, lower_r= 0, zero= 0;
  int dir_mode= -1, file_mode= -1, count, partition_number;
  int allow_dir_id_ext= -1;
@@ -1592,6 +1604,8 @@ illegal_c:;
    } else if(strcmp(argpt, "-joliet-long")==0) {
      Xorriso_relax_compliance(xorriso,
                               "joliet_long_paths:joliet_long_names", 0);
+   } else if(strcmp(argpt, "-joliet-utf16")==0) {
+     Xorriso_relax_compliance(xorriso, "joliet_utf16", 0);
    } else if(strcmp(argpt, "-fat") == 0) {
      /* was already handled in first argument scan */;
    } else if(strcmp(argpt, "-hfs-bless") == 0 ||
@@ -1779,7 +1793,8 @@ not_enough_args:;
              strcmp(argpt, "-partition_sec_hd") == 0 ||
              strcmp(argpt, "-partition_cyl_align") == 0 ||
              strcmp(argpt, "-isohybrid-mbr") == 0 ||
-             strcmp(argpt, "--grub2-mbr") == 0) {
+             strcmp(argpt, "--grub2-mbr") == 0 ||
+             strncmp(argpt, "-hppa-", 6) == 0) {
      if(i+1>=argc)
        goto not_enough_args;
      delay_opt_list[delay_opt_count++]= i;
@@ -2409,6 +2424,21 @@ problem_handler_2:;
      if(ret <= 0)
        goto problem_handler_boot;
 
+   } else if(strncmp(argpt, "-hppa-", 6) == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i++;
+     sprintf(sfe, "-as mkisofs %s %s", argpt, argv[i]);
+     palohdrversion= (xorriso->system_area_options >> 2) & 0x3f;
+     if(palohdrversion != 4)
+       palohdrversion= 5;
+     ret= Xorriso_coordinate_system_area(xorriso, palohdrversion, 0, sfe, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Xorriso_set_hppa_boot_parm(xorriso, argv[i], argpt + 6, 0);
+     if(ret <= 0)
+       goto problem_handler_boot;
+
    } else if(strcmp(argpt, "-hfs-bless") == 0) {
      static char *bless_arg_data[6]= {
         "/", "-disk_path", "", "-exec", "set_hfs_bless", "p"};
