@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -436,22 +436,15 @@ ex:;
 }
 
 
-/* @param flag bit0= delete filter with the given name
-*/
-int Xorriso_external_filter(struct XorrisO *xorriso,
-                            char *name, char *options, char *path, 
-                            int argc, char **argv, int flag)
+int Xorriso_external_filter_banned(struct XorrisO *xorriso, char *purpose,
+                                   int flag)
 {
- int ret, delete= 0, behavior= 0, extf_flag= 0, is_banned= 0;
- char *what, *what_next, *suffix= "";
- struct Xorriso_lsT *lst;
- struct Xorriso_extF *found_filter, *new_filter= NULL;
+ int is_banned= 0;
 
 #ifndef Xorriso_allow_external_filterS
  /* To be controlled by: configure --enable-external-filters */
 
- sprintf(xorriso->info_text, "%s : Banned at compile time.",
-         flag & 1 ? "-unregister_filter" : "-external_filter");
+ sprintf(xorriso->info_text, "%s : Banned at compile time.", purpose);
  Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
  sprintf(xorriso->info_text,
 "This may be changed at compile time by ./configure option --enable-external-filters");
@@ -474,16 +467,31 @@ int Xorriso_external_filter(struct XorrisO *xorriso,
  }
 #endif /* ! Xorriso_allow_extf_suiD */
 
- if(is_banned)
-   return(0);
-
  if(xorriso->filter_list_closed) {
    sprintf(xorriso->info_text,
-           "%s : Banned by previous command -close_filter_list",
-           flag & 1 ? "-unregister_filter" : "-external_filter");
+           "%s : Banned by previous command -close_filter_list", purpose);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(0);
+   is_banned= 1;
  } 
+ return(is_banned);
+}
+
+
+/* @param flag bit0= delete filter with the given name
+*/
+int Xorriso_external_filter(struct XorrisO *xorriso,
+                            char *name, char *options, char *path, 
+                            int argc, char **argv, int flag)
+{
+ int ret, delete= 0, behavior= 0, extf_flag= 0, is_banned= 0;
+ char *what, *what_next, *suffix= "";
+ struct Xorriso_lsT *lst;
+ struct Xorriso_extF *found_filter, *new_filter= NULL;
+
+ is_banned= Xorriso_external_filter_banned( xorriso,
+                      flag & 1 ? "-unregister_filter" : "-external_filter", 0);
+ if(is_banned)
+   return(0);
  if((!(flag & 1)) && path[0] != '/') {
    sprintf(xorriso->info_text,
            "-external_filter : Given command path does not begin by '/' : ");
