@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -58,7 +58,7 @@ int Exprtest_destroy(struct ExprtesT **ftest, int flag)
  if(f==NULL)
    return(0);
 
- if(f->test_type == 1) {
+ if(f->test_type == 1 || f->test_type == 13 || f->test_type == 16) {
    if(f->arg1 != NULL)
      free(f->arg1);
    if(f->arg2 != NULL) {
@@ -95,7 +95,7 @@ int Exprnode_new(struct ExprnodE **fnode, struct FindjoB *job,
  *fnode= n= TSOB_FELD(struct ExprnodE,1);
  if(n == NULL)
    return(-1);
- for(i= 0; i < sizeof(n->origin); i++)
+ for(i= 0; i < (int) sizeof(n->origin); i++)
    n->origin[i]= 0;
  strncpy(n->origin, origin, sizeof(n->origin) - 1);
  n->up= up;
@@ -367,6 +367,7 @@ int Findjob_new(struct FindjoB **o, char *start_path, int flag)
  m->use_shortcuts= 1;
  m->action= 0; /* print */
  m->prune= 0;
+ m->use_pattern= 1;
  m->target= NULL; /* a mere pointer, not managed memory */
  m->text_2= NULL; /* a mere pointer, not managed memory */
  m->user= 0;
@@ -382,6 +383,7 @@ int Findjob_new(struct FindjoB **o, char *start_path, int flag)
  m->subjob= NULL;
  m->errmsg[0]= 0;
  m->errn= 0;
+ m->match_count= 0;
 
  ret= Exprnode_new(&(m->test_tree), m, NULL, "-find", (m->use_shortcuts)<<1);
  if(ret<=0)
@@ -406,6 +408,8 @@ int Findjob_destroy(struct FindjoB **o, int flag)
    Exprnode_destroy(&(m->test_tree), 0); 
  if(m->start_path != NULL)
    free(m->start_path);
+ if(m->found_path != NULL)
+   free(m->found_path);
  free((char *) *o);
  *o= NULL;
  return(1);
@@ -497,6 +501,7 @@ int Findjob_new_node(struct FindjoB *job, struct ExprnodE **fnode,
 /* If an operator is expected : use -and
    @param flag bit0= prepare for a pseudo-test:
                      if an operator is expected, do nothing and return 2
+               bit1= use -or rather than -and
 */
 int Findjob_default_and(struct FindjoB *o, int flag)
 {
@@ -505,7 +510,11 @@ int Findjob_default_and(struct FindjoB *o, int flag)
  if(Findjob_cursor_complete(o, 0)) {
    if(flag & 1) 
      return(2);
-   ret= Findjob_and(o, 0);
+   if(flag & 2) {
+     ret= Findjob_or(o, 0);
+   } else {
+     ret= Findjob_and(o, 0);
+   }
    if(ret <= 0)
      return(ret);
  }
@@ -756,40 +765,52 @@ improper_range:;
 }
 
 
-/* @param flag bit0-1=  0= -name , 1= -wholename , 2= -disk_name
+/* @param flag bit0-1: 0= -name , 1= -wholename , 2= -disk_name , 3= -disk_path
 */
 int Findjob_set_name_expr(struct FindjoB *o, char *name_expr, int flag)
 {
- char regexpr[2*SfileadrL+2];
+ char *regexpr= NULL;
  regex_t *name_re;
  struct ExprtesT *t;
  int ret;
-   
+
+ regexpr= TSOB_FELD(char, 2*SfileadrL+2);
+ if(regexpr == NULL)
+   {ret= -1; goto ex;}
  if(strlen(name_expr)>=SfileadrL)
-   return(0);
+   {ret= 0; goto ex;};
 
  ret= Findjob_default_and(o, 0);
  if(ret <= 0)
-   return(ret);
+   goto ex;
  t= o->cursor->test;
  t->test_type= 1;
  if ((flag & 3) == 1)
    t->test_type= 13;
  else if((flag & 3) == 2)
    t->test_type= 16;
+ else if((flag & 3) == 3)
+   t->test_type= 20;
+ t->arg1= strdup(name_expr);
+ if(t->arg1 == NULL)
+   {ret= -1; goto ex;};
+
+ if((flag & 3) == 3)
+   {ret= 1; goto ex;}
+
  name_re= (regex_t *) calloc(1, sizeof(regex_t));
  if(name_re == NULL)
-   return(-1);
- t->arg1= strdup(name_expr);
- if(t->arg1 == NULL) {
-   free((char *) name_re);
-   return(-1);
- }
+   {ret= -1; goto ex;};
  Xorriso__bourne_to_reg(name_expr, regexpr, 0);
- if(regcomp(name_re, regexpr, 0) != 0)
-   return(0);
+ if(regcomp(name_re, regexpr, 0) != 0) {
+   free((char *) name_re);
+   {ret= 0; goto ex;};
+ }
  t->arg2= name_re;
- return(1);
+ ret= 1;
+ex:;
+ Xorriso_free_meM(regexpr);
+ return(ret);
 }
 
 
@@ -849,8 +870,8 @@ int Findjob_set_damage_filter(struct FindjoB *o, int value, int flag)
 }
  
 
-int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
-                          int flag)
+int Findjob_set_num_filter(struct FindjoB *o, int test_type,
+                           int num1, int num2, int flag)
 {
  struct ExprtesT *t;
  int ret;
@@ -860,20 +881,31 @@ int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
    return(ret);
 
  t= o->cursor->test;
- t->test_type= 4;
+ t->test_type= test_type;
  t->arg1= calloc(sizeof(int), 1);
  t->arg2= calloc(sizeof(int), 1);
  if(t->arg1 == NULL || t->arg2 == NULL)
    return(-1);
- *((int *) t->arg1)= start_lba;
- if(start_lba > 0)
-   *((int *) t->arg2)= start_lba + count - 1;
- else
-   *((int *) t->arg2)= start_lba - count + 1;
+ *((int *) t->arg1)= num1;
+ *((int *) t->arg2)= num2;
  return(1);
 }
 
 
+int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
+                          int flag)
+{
+ int ret, end_lba;
+
+ if(start_lba > 0)
+   end_lba= start_lba + count - 1;
+ else
+   end_lba= start_lba - count + 1;
+ ret= Findjob_set_num_filter(o, 4, start_lba, end_lba, 0);
+ return(ret);
+}
+
+
 int Findjob_set_test_hidden(struct FindjoB *o, int mode, int flag)
 {
  struct ExprtesT *t;
@@ -938,6 +970,51 @@ int Findjob_set_filter_filter(struct FindjoB *o, int value, int flag)
 }
 
 
+int Findjob_set_crtp_filter(struct FindjoB *o, char *creator, char *hfs_type,
+                            int flag)
+{
+ struct ExprtesT *t;
+ int ret;
+
+ ret= Findjob_default_and(o, 0);
+ if(ret <= 0)
+   return(ret);
+
+ t= o->cursor->test;
+ t->test_type= 18;
+ t->arg1= calloc(1, strlen(creator) + 1);
+ t->arg2= calloc(1, strlen(hfs_type) + 1);
+ if(t->arg1 == NULL || t->arg2 == NULL)
+   return(-1);
+ strcpy(t->arg1, creator);
+ strcpy(t->arg2, hfs_type);
+ return(1);
+}
+
+
+int Findjob_set_bless_filter(struct XorrisO *xorriso, struct FindjoB *o,
+                             char *blessing, int flag)
+{
+ struct ExprtesT *t;
+ int ret;
+
+ ret= Findjob_default_and(o, 0);
+ if(ret <= 0)
+   return(ret);
+
+ t= o->cursor->test;
+ t->test_type= 19;
+ t->arg1= calloc(1, sizeof(int));
+ if(t->arg1 == NULL)
+   return(-1);
+ ret= Xorriso_hfsplus_bless(xorriso, "", NULL, blessing, 4 | 8);
+ if(ret <= 0)
+   return(ret);
+ *((int *) t->arg1)= ret - 1;
+ return(1);
+}
+
+
 int Findjob_set_wanted_node(struct FindjoB *o, void *wanted_node, int flag)
 {
  struct ExprtesT *t;
@@ -967,18 +1044,19 @@ int Findjob_set_commit_filter_2(struct FindjoB *o, int flag)
 }
 
 
-int Findjob_set_decision(struct FindjoB *o, char *decision, int flag)
+int Findjob_set_arg1(struct FindjoB *o, int test_type, char *arg1, int flag)
 {
  struct ExprtesT *t;
- int ret;
+ int ret, hflag= 0;
 
- ret= Findjob_default_and(o, 0);
+ if(test_type == 23)
+   hflag= 2; /* prepend -or rather than -and */
+ ret= Findjob_default_and(o, hflag);
  if(ret <= 0)
    return(ret);
-
  t= o->cursor->test;
- t->test_type= 11;
- t->arg1= strdup(decision);
+ t->test_type= test_type;
+ t->arg1= strdup(arg1);
  if(t->arg1 == NULL)
    return(-1);
  return(1);
