@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -46,6 +46,12 @@ struct ExprtesT {
   15= -has_md5
   16= -disk_name char *arg1 (regex_t in *arg2)
   17= -hidden int *arg1 (bit0=iso_rr, bit1=joliet)
+  18= -has_hfs_crtp char *creator char *type 
+  19= -has_hfs_bless int bless_index
+  20= -disk_path char *arg1
+  21= -bad_outname int namespace
+  22= -use_pattern char *arg1 ("on" [or "ls"], "off")
+  23= -or_use_pattern char *arg1 ("on" [or "ls"], "off")
  */
  int test_type;
 
@@ -159,9 +165,17 @@ struct FindjoB {
    42= rm_merge
    43= clear_merge
    44= list_extattr
+   45= set_hfs_crtp creator type
+   46= get_hfs_crtp
+   47= set_hfs_bless blessing
+   48= get_hfs_bless
+   49= internal: update creator, type, and blessings from persistent isofs.*
+   50= print_outname namespace
+   51= report_sections
  */
  int action;
  int prune;
+ int use_pattern;
 
  /* action specific parameters */
  char *target;
@@ -186,6 +200,10 @@ struct FindjoB {
              -4 = unsupported command
              -5 = -then -elseif -else -endif without -if or at wrong place
            */
+
+ /* Counts the test matches */
+ unsigned long match_count;
+
 };
 
 
@@ -206,6 +224,9 @@ int Findjob_get_start_path(struct FindjoB *o, char **start_path, int flag);
 
 int Findjob_set_commit_filter_2(struct FindjoB *o, int flag);
 
+int Findjob_set_num_filter(struct FindjoB *o, int test_type,
+                           int num1, int num2, int flag);
+
 int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count, 
                           int flag);
 
@@ -217,8 +238,14 @@ int Findjob_set_damage_filter(struct FindjoB *o, int value, int flag);
 
 int Findjob_set_test_hidden(struct FindjoB *o, int mode, int flag);
 
+int Findjob_set_crtp_filter(struct FindjoB *o, char *creator, char *hfs_type,
+                            int flag);
 
-int Findjob_set_decision(struct FindjoB *o, char *decision, int flag);
+int Findjob_set_bless_filter(struct XorrisO *xorriso, struct FindjoB *o,
+                             char *blessing, int flag);
+
+
+int Findjob_set_arg1(struct FindjoB *o, int test_type, char *arg1, int flag);
 
 int Findjob_open_bracket(struct FindjoB *job, int flag);
 
