@@ -211,40 +211,6 @@ int Xorriso_is_isohybrid(struct XorrisO *xorriso, IsoFile *bootimg_node,
 }
 
 
-/* @return <0 yes , 0 no , <0 error */
-int Xorriso_is_grub2_elto(struct XorrisO *xorriso, IsoFile *bootimg_node,
-                          int flag)
-{
-#define Xorriso_grub2_boot_info_poS 2548
- int ret, i;
- unsigned char buf[Xorriso_grub2_boot_info_poS + 8];
- void *data_stream= NULL;
- off_t blk;
-
- ret= Xorriso_iso_file_open(xorriso, "", (void *) bootimg_node,
-                            &data_stream, 1);
- if(ret <= 0)
-   return(-1);
- ret= Xorriso_iso_file_read(xorriso, data_stream, (char *) buf,
-                            Xorriso_grub2_boot_info_poS  + 8, 0);
- Xorriso_iso_file_close(xorriso, &data_stream, 0);
- if(ret < Xorriso_grub2_boot_info_poS + 8)
-   return(0);
-
- /* >>> ??? Check for some id */;
-
- /* Check whether it has grub2_boot_info patching */
- blk= 0;
- for(i= 0; i < 8; i++)
-   blk|= buf[Xorriso_grub2_boot_info_poS + i] << (8 * i);
- blk-= 5;
- blk/= 4;
- if(blk != (off_t) xorriso->loaded_boot_bin_lba)
-   return(0);
- return(1);
-}
-
-
 int Xorriso_image_has_md5(struct XorrisO *xorriso, int flag)
 {
  int ret;
@@ -418,7 +384,7 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
  no_defaults= flag & 1;
  line= xorriso->result_line;
 
- if(flag & 16) {
+ if((flag & 16) && bin_path[0] != 0) {
    /* Allow to concentrate boot options. */
    memset(zeros, 0, 28);
    if(memcmp(id_string, zeros, 28) == 0 &&
@@ -556,15 +522,10 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
 {
  int i, num_boots, sa_type;
  char *paths[15], *line;
-
-#ifdef Libisofs_enable_unreleased_hppa_palO
-
  int ret;
  char num[4];
  char *cmdline, *bootloader, *kernel_32, *kernel_64, *ramdisk;
 
-#endif
-
  line= xorriso->result_line;
 
  sa_type= (xorriso->system_area_options & 0xfc) >> 2;
@@ -598,9 +559,6 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
    return(num_boots);
  }
  if(sa_type == 4 || sa_type == 5) {
-
-#ifdef Libisofs_enable_unreleased_hppa_palO
-
    ret= iso_image_get_hppa_palo(image, &cmdline, &bootloader, &kernel_32,
                                 &kernel_64, &ramdisk);
    if(ret == 1) {
@@ -612,9 +570,6 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
      sprintf(num, "%d", sa_type);
      Xorriso_status_hppa(xorriso, "hdrversion", num, filter, fp, 0);
    }
-
-#endif /* ! Libisofs_enable_unreleased_hppa_palO */
-
    return(0);
  }
  return(0);
@@ -651,7 +606,7 @@ int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
                               int flag)
 {
  int ret, i, num_boots, hflag;
- int bin_path_in_use= 0, is_default, no_defaults;
+ int is_default, no_defaults;
  char *path= NULL, *form= "any", *line, *hpt;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
@@ -697,9 +652,6 @@ no_image:;
    goto report_open_item;
  }
 
- if(xorriso->boot_image_bin_path[0] || xorriso->boot_count > 0)
-   bin_path_in_use= 1;
-
  is_default= (xorriso->boot_image_cat_path[0] == 0);
  sprintf(line,"-boot_image %s cat_path=", form);
  Text_shellsafe(xorriso->boot_image_cat_path, line, 1);
@@ -825,11 +777,8 @@ int Xorriso__append_boot_params(char *line, ElToritoBootImage *bootimg,
 */
 int Xorriso_show_boot_info(struct XorrisO *xorriso, int flag)
 {
- int ret, bin_path_valid= 0,has_isolinux_mbr= 0, i, num_boots, has_mbr= 0;
- int has_grub2_mbr= 0;
- unsigned int mbr_lba= 0;
- off_t lb0_count, blk;
- char *respt, *path;
+ int ret, bin_path_valid= 0, i, num_boots, sa_count;
+ char *respt, *path= NULL, **sa_report= NULL, *sa_summary= NULL;
  unsigned char *lb0= NULL;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
@@ -845,7 +794,7 @@ int Xorriso_show_boot_info(struct XorrisO *xorriso, int flag)
 
  if(xorriso->boot_count > 0) {
    if(!(flag & 1)) {
-     sprintf(respt, "Boot record  : overridden by -boot_image any next\n");
+     sprintf(respt, "Boot record  : (overridden by -boot_image any next)\n");
      Xorriso_toc_line(xorriso, flag & 8);
    }
    ret= 1; goto ex;
@@ -866,11 +815,27 @@ no_boot:;
    goto ex;
  }
 
+ ret= iso_image_report_system_area(image, &sa_report, &sa_count, 0);
+ if(ret > 0 && sa_report != NULL)
+   for(i= 0; i < sa_count; i++)
+     if(strncmp(sa_report[i], "System area summary: ", 21) == 0) {
+       Xorriso_alloc_meM(sa_summary, char, strlen(sa_report[i] + 21) + 1);
+       strcpy(sa_summary, sa_report[i] + 21);
+   break;
+     }
+ if(sa_report != NULL)
+   iso_image_report_system_area(image, &sa_report, &sa_count, 1 << 15);
+ Xorriso_process_msg_queues(xorriso,0);
+
  /* Using the nodes with extreme care . They might be deleted meanwhile. */
  ret= iso_image_get_boot_image(image, &bootimg, &bootimg_node, &bootcat_node);
- if(ret != 1)
-   goto no_boot;
-
+ if(ret != 1) {
+   if(sa_summary == NULL)
+     goto no_boot;
+   sprintf(respt, "Boot record  : (system area only) , %s\n", sa_summary);
+   Xorriso_toc_line(xorriso, flag & 8);
+   ret= 1; goto ex;
+ }
  ret= iso_image_get_all_boot_imgs(image, &num_boots, &boots, &bootnodes, 0);
  Xorriso_process_msg_queues(xorriso,0);
  if(ret != 1) {
@@ -881,59 +846,9 @@ no_boot:;
      bin_path_valid= 1;
  }
  sprintf(respt, "Boot record  : El Torito");
- /* Load and examine potential MBR */
- ret= burn_read_data(drive, (off_t) 0, (char *) lb0, (off_t) 2048,
-                     &lb0_count, 2);
- if(ret > 0)
-   if(lb0[510] == 0x55 && lb0[511] == 0xaa)
-     has_mbr= 1;
- if(bin_path_valid)
-   ret= Xorriso_is_isohybrid(xorriso, bootimg_node, 0);
- else
-   ret= 0;
- if(ret > 0) {
-   if(has_mbr) {
-     has_isolinux_mbr= 1;
-     mbr_lba= lb0[432] | (lb0[433] << 8) | (lb0[434] << 16) | (lb0[435] << 24);
-     mbr_lba/= 4;
-     if(mbr_lba != (unsigned int) xorriso->loaded_boot_bin_lba)
-       has_isolinux_mbr= 0;
-     if(has_isolinux_mbr) {
-       for(i= 0; i < 426; i++)
-         if(strncmp((char *) (lb0 + i), "isolinux", 8) == 0)
-       break;
-       if(i >= 426)
-         has_isolinux_mbr= 0;
-     }
-   }
-   if(has_isolinux_mbr)
-     strcat(respt, " , ISOLINUX isohybrid MBR pointing to boot image");
-   else
-     strcat(respt, " , ISOLINUX boot image capable of isohybrid");
- }
- if(bin_path_valid)
-   ret= Xorriso_is_grub2_elto(xorriso, bootimg_node, 0);
- else
-   ret= 0;
- if(ret > 0) {
-   if(has_mbr) {
-     has_grub2_mbr= 1;
-     blk= 0;
-     for(i= 0; i < 8; i++)
-       blk|= lb0[i + 0x1b0] << (8 * i);
-     blk-= 4;
-     blk/= 4;
-     if(blk != (off_t) xorriso->loaded_boot_bin_lba)
-       has_grub2_mbr= 0;
-     for(i= 32; i < 427; i++)
-       if(strncmp((char *) (lb0 + i), "GRUB", 4) == 0)
-     break;
-     if(i >= 427)
-       has_grub2_mbr= 0;
-   }
-   if(has_grub2_mbr)
-     strcat(respt, " , GRUB2 MBR pointing to first boot image");
- }
+ if(sa_summary != NULL)
+   sprintf(respt + strlen(respt), " , %s", sa_summary);
+
  strcat(respt, "\n");
  Xorriso_toc_line(xorriso, flag & 8);
  if(flag & 2)
@@ -984,6 +899,7 @@ ex:;
    iso_image_unref(image); /* release obtained reference */
  Xorriso_free_meM(path);
  Xorriso_free_meM(lb0);
+ Xorriso_free_meM(sa_summary);
  return(ret);
 } 
 
@@ -1176,7 +1092,7 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
  if(strstr(devadr, "mmc:") == devadr)
    devadr+= 4;
  ret= Xorriso_make_mount_cmd(xorriso, cmd, lba, track, session, volid, devadr,
-                             mount_command, flag & (2 | 4));
+                          mount_command, (flag & (2 | 4)) | ((flag & 4) << 1));
  if(ret <= 0)
    goto ex;
  if(ret == 2)
@@ -1210,7 +1126,8 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
      sprintf(xorriso->result_line, "%s\n", mount_command);
      Xorriso_result(xorriso,0);
    } else {
-     ret= Xorriso_execv(xorriso, mount_command, "/bin:/sbin", &status, 1);
+     ret= Xorriso_execv(xorriso, mount_command, 0, NULL, "/bin:/sbin",
+                        NULL, NULL, NULL, &status, 1);
      if(WIFEXITED(status) && WEXITSTATUS(status) != 0) {
        sprintf(xorriso->info_text,
                "-mount : mount command failed with exit value %d",
@@ -1302,14 +1219,8 @@ int Xorriso_set_hppa_boot_parm(struct XorrisO *xorriso, char *text, char *what,
  par[0]= par[1]= par[2]= par[3]= par[4]= NULL;
  if(flag & 1) {
    /* Give up HP-PA boot parameters */
-
-#ifdef Libisofs_enable_unreleased_hppa_palO
-
    iso_image_set_hppa_palo(image, par[0], par[1], par[2], par[3], par[4],
                            1);
-
-#endif
-
    return(1);
  }
  if(strcmp(what, "cmdline") == 0) {
@@ -1343,9 +1254,6 @@ int Xorriso_set_hppa_boot_parm(struct XorrisO *xorriso, char *text, char *what,
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
  }
-
-#ifdef Libisofs_enable_unreleased_hppa_palO
-
  ret= iso_image_set_hppa_palo(image, par[0], par[1], par[2], par[3], par[4],
                               0);
  if (ret < 0) {
@@ -1355,43 +1263,50 @@ int Xorriso_set_hppa_boot_parm(struct XorrisO *xorriso, char *text, char *what,
    return(0);
  }
  return(1);
-
-#else
-
- strcpy(xorriso->info_text, "HP-PA boot preparations are not yet supported");
- Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
- return(0);
-
-#endif /* ! Libisofs_enable_unreleased_hppa_palO */
-
 }
 
 
+/* @param flag bit0= do not set xorriso->system_area_options, just check
+               bit1= only check for grub2_mbr <-> isolinux partition_table
+*/
 int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
                                    int options, char *cmd, int flag)
 {
- int old_type, old_options;
- static char *type_names[4] = {
+ int old_type, old_options, new_options;
+ static char *type_names[6] = {
       "MBR", "MIPS Big Endian Volume Header", "MIPS Little Endian Boot Block",
-      "SUN Disk Label"};
+      "SUN Disk Label", "HP-PA PALO v4", "HP-PA PALO v5"};
 
  old_type= (xorriso->system_area_options & 0xfc) >> 2;
- old_options= xorriso->system_area_options & ~0x40fc;
+ old_options= xorriso->system_area_options & 0x3c03;
+ new_options= options & 0x3c03;
+ if(((options & (1 << 14)) && (xorriso->system_area_options & 2)) ||
+    ((options & 2) && (xorriso->system_area_options & (1 << 14))))
+   goto reject;
+ if(flag & 2)
+   return(1);
  if((old_type != 0 || old_options != 0) &&
-    (old_type != sa_type || (old_options != 0 && old_options != options))) {
+    (old_type != sa_type || (old_options != 0 && old_options != new_options))){
+reject:;
    sprintf(xorriso->info_text, "%s : First sector already occupied by %s",
-           cmd, old_type < 4 ? type_names[old_type] : "other boot facility");
-   if(old_type == 0 && old_options == 2)
+           cmd, old_type < 6 ? type_names[old_type] : "other boot facility");
+   if(old_type == 0 && (old_options & 2))
      strcat(xorriso->info_text, " for ISOLINUX isohybrid");
-   if(old_type == 0 && old_options == 1)
+   else if (old_type == 0 && (xorriso->system_area_options & (1 << 14))) {
+     strcat(xorriso->info_text, " for GRUB2 patching");
+     if(old_type == 0 && (old_options & 1))
+       strcat(xorriso->info_text, " with partition table");
+   } else if(old_type == 0 && (old_options & 1))
      strcat(xorriso->info_text, " for partition table");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    goto hint_revoke;
  }
- xorriso->system_area_options= (xorriso->system_area_options & 0x4000) |
-                               ((sa_type << 2) & 0xfc) | (options & ~0x40fc);
- if(sa_type == 0)
-   xorriso->patch_system_area= xorriso->system_area_options;
+ if(!(flag & 1)) {
+   xorriso->system_area_options= (xorriso->system_area_options & ~0x3cff) |
+                                 ((sa_type << 2) & 0xfc) | (options & 0x3c03);
+   if(sa_type == 0)
+     xorriso->patch_system_area= xorriso->system_area_options;
+ }
  return(1);
 
 hint_revoke:;
@@ -1406,3 +1321,128 @@ hint_revoke:;
  return(0);
 }
 
+
+int Xorriso_gpt_crc(struct XorrisO *xorriso, char *path, int flag)
+{
+ int ret;
+ char *buf = NULL;
+ FILE *fp = NULL;
+ uint32_t crc;
+
+ Xorriso_alloc_meM(buf, char, 32 * 1024);
+
+ ret= Xorriso_afile_fopen(xorriso, path, "rb", &fp, 0);
+ if(ret <= 0)
+   goto ex;
+ ret= fread(buf, 1, 32 * 1024, fp);
+ if(ret == 0) {
+   strcpy(xorriso->info_text,
+          "No bytes readable for GPT CRC from ");
+   Text_shellsafe(path, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+   ret= 0; goto ex;
+ }
+ crc= iso_crc32_gpt((unsigned char *) buf, ret, 0);
+ sprintf(xorriso->result_line, "0x%8.8x\n", (unsigned int) crc);
+ Xorriso_result(xorriso, 0);
+ ret= 1;
+ex:;
+ if(fp != NULL && fp != stdin)
+   fclose(fp);
+ Xorriso_free_meM(buf);
+ return(ret);
+}
+
+
+static void Xorriso_report_lines(struct XorrisO *xorriso,
+                                char **lines, int line_count)
+{
+ int i;
+
+ for(i = 0; i < line_count ; i++) {
+   sprintf(xorriso->result_line, "%s\n", lines[i]);
+   Xorriso_result(xorriso,0);
+ }
+} 
+
+
+/* @param flag bit0= report El Torito rather than System Area
+*/
+int Xorriso_report_system_area(struct XorrisO *xorriso, char *form, int flag)
+{
+ int ret, line_count;
+ char **lines = NULL;
+ IsoImage *image;
+
+ if(strcmp(form, "help") == 0) {
+   if(flag & 1)
+     ret= iso_image_report_el_torito(NULL, &lines, &line_count, 1);
+   else
+     ret= iso_image_report_system_area(NULL, &lines, &line_count, 1);
+   if(ret <= 0)
+     goto ex;
+   sprintf(xorriso->result_line,
+"------------------------------------------------------------------------------\n");
+   Xorriso_result(xorriso, 0);
+   if(flag & 1)
+   sprintf(xorriso->result_line, "With -report_el_torito \"plain\":\n");
+   else
+     sprintf(xorriso->result_line, "With -report_system_area \"plain\":\n");
+   Xorriso_result(xorriso, 0);
+   sprintf(xorriso->result_line,
+"------------------------------------------------------------------------------\n");
+   Xorriso_result(xorriso, 0);
+   sprintf(xorriso->result_line, "\n");
+   Xorriso_result(xorriso, 0);
+
+ } else if(strcmp(form, "") == 0 || strcmp(form, "plain") == 0) {
+   ret= Xorriso_get_volume(xorriso, &image, 0);
+   if(ret <= 0)
+     goto ex;
+   if(flag & 1)
+     ret= iso_image_report_el_torito(image, &lines, &line_count, 0);
+   else
+     ret= iso_image_report_system_area(image, &lines, &line_count, 0);
+
+ } else if(strncmp(form, "gpt_crc_of:", 11) == 0 && !(flag & 1)) {
+   ret = Xorriso_gpt_crc(xorriso, form + 11, 0);
+   goto ex;
+
+ } else {
+   sprintf(xorriso->info_text,
+           "%s form parameter not recognized: ",
+           flag & 1 ? "-report_el_torito" : "-report_system_area");
+   Text_shellsafe(form, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if(ret < 0)
+   goto ex;
+ if(lines == NULL || ret == 0) {
+   if(flag & 1)
+     strcpy(xorriso->info_text, "No El Torito information was loaded");
+   else
+     strcpy(xorriso->info_text, "No System Area was loaded");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
+   ret= 2; goto ex;
+ }
+ if(line_count == 0) {
+   if(flag & 1)
+     strcpy(xorriso->info_text, "No El Torito information available");
+   else
+     strcpy(xorriso->info_text, "System Area only contains 0-bytes");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
+   ret= 2; goto ex;
+ }
+ Xorriso_report_lines(xorriso, lines, line_count);
+ ret= 1;
+ex:;
+ if(lines != NULL) {
+   if(flag & 1)
+     iso_image_report_el_torito(NULL, &lines, &line_count, 1 << 15);
+   else
+     iso_image_report_system_area(NULL, &lines, &line_count, 1 << 15);
+ }
+ return(ret);
+}
+
