@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -103,6 +103,7 @@ int Xorriso_set_signal_handling(struct XorrisO *xorriso, int flag)
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FATAL", 0);
    return(-1);
  }
+ mode|= 256; /* Ignore SIGPIPE */
 
  /* <<< */
  sprintf(xorriso->info_text, "burn_set_signal_handling(%d)", mode);
@@ -254,6 +255,11 @@ int Xorriso_detach_libraries(struct XorrisO *xorriso, int flag)
    if(xorriso->libs_are_started==0)
      return(0);
    isoburn_finish();
+
+#ifdef Xorriso_with_editlinE
+   Xorriso__shutdown_editline(0);
+#endif
+
  }
  return(1);
 }
@@ -928,10 +934,14 @@ int Xorriso_list_extras(struct XorrisO *xorriso, char *mode, int flag)
  Xorriso_list_extras_result(xorriso, mode, "dvd_obs", 0);
 
  sprintf(xorriso->result_line, "Readline     : %s\n",
+#ifdef Xorriso_with_editlinE
+         "yes , libedit");
+#else
 #ifdef Xorriso_with_readlinE
          "yes");
 #else
          "no");
+#endif
 #endif
  Xorriso_list_extras_result(xorriso, mode, "use_readline", 0);
 
