@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -667,9 +667,10 @@ unusable_size:;
 int Xorriso_option_boot_image(struct XorrisO *xorriso, char *form,
                               char *treatment, int flag)
 {
- int was_ok= 1, ret, isolinux_grub= 0, count, bin_count;
+ int was_ok= 1, ret, isolinux_grub= 0, count, bin_count, parm_len;
+ int palohdrversion;
  unsigned int u;
- char *formpt, *treatpt, *eff_path= NULL;
+ char *formpt, *treatpt, *eff_path= NULL, *eqpt, parm[20];
  uint8_t sn[8];
  double num;
 
@@ -864,9 +865,11 @@ treatment_patch:;
 
  } else if(strcmp(treatpt, "mips_discard") == 0 ||
            strcmp(treatpt, "mipsel_discard") == 0 ||
-           strcmp(treatpt, "sparc_discard") == 0) {
+           strcmp(treatpt, "sparc_discard") == 0 || 
+           strcmp(treatpt, "hppa_discard") == 0) {
    xorriso->system_area_options&= ~0xfc; /* system area type 0 */
    Xorriso_add_mips_boot_file(xorriso, "", 1); /* give up MIPS boot files */
+   Xorriso_set_hppa_boot_parm(xorriso, "", "", 1); /* give up HP-PA files */
 
  } else if(strncmp(treatpt, "sparc_label=", 12) == 0) {
    sprintf(eff_path, "-boot_image %s sparc_label=", formpt);
@@ -883,6 +886,29 @@ treatment_patch:;
    if(ret <= 0)
      goto ex;
 
+ } else if(strncmp(treatpt, "hppa_", 5) == 0) {
+   sprintf(eff_path, "-boot_image %s %s", formpt, treatpt);
+   palohdrversion= (xorriso->system_area_options >> 2) & 0x3f;
+   if(palohdrversion != 4)
+     palohdrversion= 5;
+   ret= Xorriso_coordinate_system_area(xorriso, palohdrversion, 0, eff_path,
+                                       0);
+   if(ret <= 0)
+     goto ex;
+   eqpt= strchr(treatpt, '=');
+   if(eqpt == NULL) {
+     sprintf(xorriso->info_text,
+             "No equal sign found in -boot_image %s %s", form, treatment);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+   parm_len= (eqpt - treatpt) - 5;
+   if(parm_len > (int) sizeof(parm) - 1)
+     parm_len= sizeof(parm) - 1;
+   strncpy(parm, treatpt + 5, parm_len);
+   parm[parm_len]= 0;
+   ret= Xorriso_set_hppa_boot_parm(xorriso, eqpt + 1, parm, 0);
+
  } else if(strncmp(treatpt, "boot_info_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0)
      xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
@@ -942,6 +968,7 @@ treatment_patch:;
    ret= Xorriso_set_system_area_path(xorriso, treatpt + 12, 0);
    if(ret <= 0)
      goto ex;
+   xorriso->system_area_options&= ~0x4000;
 
  } else if(strncmp(treatpt, "partition_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0) {
@@ -1149,24 +1176,17 @@ treatment_patch:;
 
 #endif /* ! Xorriso_with_isohybriD */
 
- } else if(strncmp(treatpt, "grub2_mbr=", 9) == 0 &&
-           strcmp(formpt, "grub")==0) {
-
-   if(strcmp(treatpt + 9, "off") == 0)
+ } else if(strncmp(treatpt, "grub2_mbr=", 10) == 0) {
+   ret= Xorriso_coordinate_system_area(xorriso, 0, (1 << 14), eff_path, 1 | 2);
+   if(ret <= 0)
+     goto ex;
+   ret= Xorriso_set_system_area_path(xorriso, treatpt + 10, 0);
+   if(ret <= 0)
+     goto ex;
+   if(treatpt [10] == 0)
      xorriso->system_area_options&= ~0x4000;
-   else if(strcmp(treatpt + 9, "on") == 0)
-     xorriso->system_area_options=
-                                  (xorriso->system_area_options & ~2) | 0x4000;
-   else {
-     sprintf(xorriso->info_text,
-             "Unrecognized keyword with -boot_image %s %s",
-             form, treatment);
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-     sprintf(xorriso->info_text,
-             "Allowed with grub2_mbr= are: off , on");
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "HINT", 0);
-     {ret= 0; goto ex;}
-   }
+   else 
+     xorriso->system_area_options|= 0x4000;
 
  } else
    was_ok= 0;
@@ -2157,6 +2177,80 @@ int Xorriso_option_compliance(struct XorrisO *xorriso, char *mode,
 }
 
 
+/* Command -concat */
+int Xorriso_option_concat(struct XorrisO *xorriso,
+                          int argc, char **argv, int *idx, int flag)
+{
+ int ret, end_idx, optc= 0, progc= 0, iso_rr_start, prog_end_idx= -1;
+ char **optv= NULL, **progv= NULL, *delimiter_mem= NULL;
+
+ /* Must be done before any goto ex; */
+ end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
+ iso_rr_start= *idx + 2;
+
+ if(xorriso->allow_restore <= 0) {
+   sprintf(xorriso->info_text,
+          "-concat: image content copies are not enabled by option -osirrox");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if(end_idx - *idx < 3) {
+   sprintf(xorriso->info_text,
+           "-concat: Not enough arguments. Need mode, target, iso_rr_path.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if(strcmp(argv[*idx], "pipe") == 0) {
+   if(end_idx - *idx < 5) {
+     sprintf(xorriso->info_text,
+             "-concat pipe: Not enough arguments. Need delimiter, program path, delimiter, iso_rr_path.");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+
+   Xorriso_alloc_meM(delimiter_mem, char, strlen(xorriso->list_delimiter) + 1);
+   strcpy(delimiter_mem, xorriso->list_delimiter);
+   strcpy(xorriso->list_delimiter, argv[*idx + 1]);
+   ret= Xorriso_opt_args(xorriso, "-concat pipe", argc , argv, *idx + 2,
+                         &prog_end_idx, &progc, &progv, 4 | 128);
+   strcpy(xorriso->list_delimiter, delimiter_mem);
+   if(ret <= 0)
+     goto ex;
+   if(progc <= 0) {
+     sprintf(xorriso->info_text, "-concat pipe: No program path given.");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+   iso_rr_start= prog_end_idx + 1;
+ }
+ ret= Xorriso_opt_args(xorriso, "-concat", argc , argv, iso_rr_start, &end_idx,
+                       &optc, &optv, 128);
+ if(ret <= 0)
+   goto ex;
+ if(optc <= 0) {
+   sprintf(xorriso->info_text, "-concat: No iso_rr_paths given.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+   ret= 0; goto ex;
+ }
+ ret= Xorriso_concat(xorriso, argv[*idx], argv[*idx + 1],
+                     progc, progv, optc, optv, 0);
+ex:;
+ if(progv != NULL) {
+   if(delimiter_mem != NULL)
+      strcpy(xorriso->list_delimiter, argv[*idx + 2]);
+   Xorriso_opt_args(xorriso, "-concat", argc, argv, *idx + 2, &prog_end_idx,
+                    &progc, &progv, 256);
+   if(delimiter_mem != NULL)
+     strcpy(xorriso->list_delimiter, delimiter_mem);
+ }
+ Xorriso_opt_args(xorriso, "-concat", argc, argv, iso_rr_start, &end_idx,
+                  &optc, &optv, 256);
+ Xorriso_free_meM(delimiter_mem);
+ *idx= end_idx;
+ return(ret); 
+}
+
+
 /* Option -copyright_file */
 int Xorriso_option_copyright_file(struct XorrisO *xorriso, char *name, int flag)
 {
