@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -29,6 +29,12 @@
 #include "xorriso_private.h"
 #include "xorrisoburn.h"
 
+#ifdef Xorriso_with_readlinE
+#define Xorriso_with_line_editoR
+#endif
+#ifdef Xorriso_with_editlinE
+#define Xorriso_with_line_editoR
+#endif
 
 /* Command -data_cache_size */
 int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
@@ -660,6 +666,23 @@ ex:;
 }
 
 
+static int Xorriso_determine_name_space(struct XorrisO *xorriso,
+                                        char *space_name, int flag)
+{
+ if(strcmp(space_name, "rockridge") == 0)
+   return(1);
+ else if(strcmp(space_name, "joliet") == 0)
+   return(2);
+ else if(strcmp(space_name, "ecma119") == 0 ||
+         strcmp(space_name, "iso9660") == 0)
+   return(3);
+ else if(strcmp(space_name, "hfsplus") == 0)
+   return(4);
+ sprintf(xorriso->info_text, "-find: Unknown output namespace identifier");
+ return(0);
+}
+
+
 /* Option -find alias -findi, and -findx */
 /* @param flag bit0= -findx rather than -findi
                bit1= do not reset pacifier, no final pacifier message
@@ -673,7 +696,7 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag)
 {
  int ret, i, end_idx, type= 0, action, deleter= 0, start_lba, count;
- int list_extattr_head= 0, bsl_mem, disk_path;
+ int list_extattr_head= 0, bsl_mem, disk_path, name_space;
  struct FindjoB *job, *first_job= NULL, *new_job;
  char *start_path, *path= NULL, *cpt, *other_path_start= NULL, *cd_pt;
  char *access_acl_text= NULL, *default_acl_text= NULL, *list_extattr_mode;
@@ -703,6 +726,7 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
  if(!(flag&2))
    xorriso->find_compare_result= 1;
  for(i= *idx+1; i<end_idx; i++) {
+   ret= 1;
    if(strcmp(argv[i], "-name")==0) {
      if(i+1>=end_idx) {
 not_enough_arguments:;
@@ -831,43 +855,94 @@ not_enough_arguments:;
        sprintf(xorriso->info_text, "-findi: cannot setup -has_hfs_bless test");
        goto sorry_ex;
      }
+   } else if(strcmp(argv[i], "-bad_outname")==0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i+= 1;
+     name_space= Xorriso_determine_name_space(xorriso, argv[i], 0);
+     if(name_space < 0) {
+       ret= 0; goto sorry_ex;
+     }
+     ret= Findjob_set_num_filter(job, 21, name_space, 0, 0);
+     if(ret <= 0) {
+       sprintf(xorriso->info_text, "-findi: cannot setup -bad_outname test");
+       goto sorry_ex;
+     }
    } else if(strcmp(argv[i], "-true") == 0) {
      ret= Findjob_set_false(job, -1, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-false") == 0) {
      ret= Findjob_set_false(job, 1, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-decision") == 0) {
      if(i+1>=end_idx)
        goto not_enough_arguments;
      i++;
-     ret= Findjob_set_decision(job, argv[i], 0);
+     ret= Findjob_set_arg1(job, 11, argv[i], 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-prune") == 0) {
      ret= Findjob_set_prune(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-sub") == 0 || strcmp(argv[i], "(") == 0) {
      ret= Findjob_open_bracket(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-subend") == 0 || strcmp(argv[i], ")") == 0) {
      ret= Findjob_close_bracket(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-not") == 0 || strcmp(argv[i], "!") == 0) {
      ret= Findjob_not(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-and") == 0 || strcmp(argv[i], "-a") == 0) {
      ret= Findjob_and(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-or") == 0 || strcmp(argv[i], "-o") == 0) {
      ret= Findjob_or(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-if") == 0) {
      ret= Findjob_if(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-then") == 0) {
      ret= Findjob_then(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-else") == 0) {
      ret= Findjob_else(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-elseif") == 0) {
      ret= Findjob_elseif(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-endif") == 0) {
      ret= Findjob_endif(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-sort_lba") == 0) {
      flag|= 8;
      /* If an operator is open: insert a -true test, else do nothing */
      ret= Findjob_set_false(job, -1, 1);
      if(ret == 2)
        ret= 1;
+   } else if(strcmp(argv[i], "-use_pattern") == 0 ||
+             strcmp(argv[i], "-or_use_pattern") == 0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i++;
+     ret= Findjob_set_arg1(job,
+                           22 + (strcmp(argv[i - 1], "-or_use_pattern") == 0),
+                           argv[i], 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-exec")==0) {
      if(i+1>=end_idx) {
 not_enough_exec_arguments:;
@@ -1139,6 +1214,17 @@ not_enough_exec_arguments:;
        Findjob_set_action_target(job, 47, argv[i], 0);
      } else if(strcmp(cpt, "get_hfs_bless")==0) {
        Findjob_set_action_target(job, 48, NULL, 0);
+     } else if(strcmp(cpt, "print_outname")==0) {
+       if(i+1>=end_idx)
+         goto not_enough_exec_arguments;
+       i++;
+       name_space= Xorriso_determine_name_space(xorriso, argv[i], 0);
+       if(name_space < 0) {
+         ret= 0; goto sorry_ex;
+       }
+       Findjob_set_action_type(job, 50, name_space, 0);
+     } else if(strcmp(cpt, "report_sections")==0) {
+       Findjob_set_action_target(job, 51, NULL, 0);
      } else {
        sprintf(xorriso->info_text, "-find -exec: unknown action ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -1226,17 +1312,19 @@ ex:;
 /* Option -follow */
 int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag)
 {
- int was_fl, was_fm, was_fpr, was_fpt, l;
+ int was_fl, was_fm, was_fpr, was_fpt, was_fc, l;
  double num;
  char *cpt, *npt;
 
  was_fpt= xorriso->do_follow_pattern;
  was_fpr= xorriso->do_follow_param;
  was_fl= xorriso->do_follow_links;
+ was_fc= xorriso->do_follow_concat;
  was_fm= xorriso->do_follow_mount;
  xorriso->do_follow_pattern= 0;
  xorriso->do_follow_param= 0;
  xorriso->do_follow_links= 0;
+ xorriso->do_follow_concat= 0;
  xorriso->do_follow_mount= 0;
  npt= cpt= mode;
  for(cpt= mode; npt!=NULL; cpt= npt+1) {
@@ -1251,16 +1339,19 @@ int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->do_follow_pattern= 0;
      xorriso->do_follow_param= 0;
      xorriso->do_follow_links= 0;
+     xorriso->do_follow_concat= 0;
      xorriso->do_follow_mount= 0;
    } else if(strncmp(cpt, "on", l)==0) {
      xorriso->do_follow_pattern= 1;
      xorriso->do_follow_param= 1;
      xorriso->do_follow_links= 1;
+     xorriso->do_follow_concat= 1;
      xorriso->do_follow_mount= 1;
    } else if(strncmp(cpt, "default", l)==0) {
      xorriso->do_follow_pattern= 1;
      xorriso->do_follow_param= 0;
      xorriso->do_follow_links= 0;
+     xorriso->do_follow_concat= 0;
      xorriso->do_follow_mount= 1;
      xorriso->follow_link_limit= 100;
    } else if(strncmp(cpt, "link", l)==0 || strncmp(cpt,"links", l)==0) {
@@ -1271,6 +1362,8 @@ int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->do_follow_param= 1;
    } else if(strncmp(cpt, "pattern", l)==0) {
      xorriso->do_follow_pattern= 1;
+   } else if(strncmp(cpt, "concat", l)==0) {
+     xorriso->do_follow_concat= 1;
    } else if(strncmp(cpt, "limit=", 6)==0) {
      sscanf(cpt+6, "%lf", &num);
      if(num<=0 || num>1.0e6) {
@@ -1290,6 +1383,7 @@ sorry_ex:
      xorriso->do_follow_pattern= was_fpt;
      xorriso->do_follow_param= was_fpr;
      xorriso->do_follow_links= was_fl;
+     xorriso->do_follow_concat= was_fc;
      xorriso->do_follow_mount= was_fm;
      return(0);
    }
@@ -1644,7 +1738,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              |\"efi_boot_part=\"|\"efi_boot_part=--efi-boot-image\"",
 "              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
 "              |\"sparc_label=\"|\"grub2_sparc_core=\"|\"sparc_discard\"",
-"              |\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
+"              |\"hppa_cmdline=\"|\"hppa_bootloader=\"|\"hppa_kernel_32=\"",
+"              |\"hppa_kernel_64=\"|\"hppa_ramdisk=\"|\"hppa_hdrversion=\"",
+"              |\"hppa_discard\"|\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
 "              |\"apm_block_size=\"|\"show_status\"",
 "                 Whether to discard or keep an exiting El Torito boot image.",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
@@ -1658,8 +1754,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 efi_boot_part= is for booting EFI systems from USB stick.",
 "                 mips_path= adds Big Endian MIPS boot files. mipsel_path=",
 "                 sets one Little Endian MIPS boot file. sparc_label=",
-"                 activates SUN Disk Label. All three are mutually exclusive",
-"                 and exclusive to MBR production.",
+"                 activates SUN Disk Label. hppa_* is for HP PA-RISC via PALO.",
+"                 All four are mutually exclusive and exclusive to production",
+"                 of MBR and to booting via EFI from USB stick.",
 "  -append_partition partition_number type_code disk_path",
 "                 Append a prepared filesystem image after the end of the",
 "                 ISO image. Caution: Will be overwritten by multi-session.",
@@ -1693,7 +1790,17 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "",
 "  -tell_media_space",
 "                 Print foreseeable available space on output medium",
-"  -pvd_info      Print various id strings of the loaded ISO image."
+"  -pvd_info      Print various id strings of the loaded ISO image.",
+"",
+"  -report_el_torito \"plain\"|\"help\"",
+"                 \"plain\" prints information about the El Torito boot catalog",
+"                 and boot images of the loaded ISO image.",
+"                 \"help\" prints an explanation of the output format.",
+"  -report_system_area \"plain\"|\"help\"|\"gpt_crc_of:\"disk_path",
+"                 \"plain\" prints information about recognized data",
+"                 in the System Area of the loaded ISO image: MBR, GPT, ...",
+"                 \"help\" prints an explanation of the output format.",
+"                 \"gpt_crc_of:\" prints GPT CRC of file disk_path.",
 "",
 "Commands with variable length path list [...] need the list delimiter text",
 "as end mark if they are followed by another command. By default this",
@@ -1802,7 +1909,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   -type b|c|d|p|f|l|s|e, -pending_data, -hidden,",
 "                   -lba_range start count, -damaged, -has_acl, -has_xattr,",
 "                   -has_aaip, -has_filter, -has_md5, -has_any_xattr,",
-"                   -has_hfs_crtp, -has_hfs_bless,",
+"                   -has_hfs_crtp, -has_hfs_bless, -bad_outname,",
 "                   -prune, -decision yes|no, -true, -false",
 "                 Operators: -not, -or, -and, -sub, (, -subend, ),",
 "                   -if, -then, -elseif, -else, -endif",
@@ -1812,7 +1919,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   getfacl, setfacl, getfattr, setfattr, get_any_xattr,",
 "                   list_extattr, get_md5, check_md5, make_md5,",
 "                   set_hfs_crtp, get_hfs_crtp, set_hfs_bless, get_hfs_bless,",
-"                   set_filter, show_stream, mkisofs_r, hide, find.",
+"                   set_filter, show_stream, mkisofs_r, hide, print_outname,",
+"                   estimate_size, find",
 "                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
 "                 Create empty directories if they do not exist yet.",
@@ -1839,7 +1947,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Like -not_list but with line rules as -dialog \"on\".",
 "  -not_mgt \"reset\"|\"on\"|\"off\"|\"param_on\"|\"subtree_on\"|\"ignore_on\"",
 "                 Control effect of exclusion lists.",
-"  -follow \"on\"|\"pattern:param:link:mount:limit=#\"|\"default\"|\"off\"",
+"  -follow \"on\"|\"pattern:param:link:concat:mount:limit=#\"|\"default\"|\"off\"",
 "                 Follow symbolic links and mount points within disk_path.",
 "  -overwrite \"on\"|\"nondir\"|\"off\"",
 "                 Allow or disallow to overwrite existing files in ISO image.",
@@ -1912,7 +2020,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Try to circumvent slow checkread on DVD-RAM, BD-RE, BD-R.",
 "  -dvd_obs \"default\"|\"32k\"|\"64k\"",
 "                 Set number of bytes per DVD/BD write operation.",
-"  -stdio_sync \"on\"|\"off\"|number",
+"  -stdio_sync \"on\"|\"off\"|\"end\"|number",
 "                 Set number of bytes after which to force output to stdio",
 "                 pseudo drives. \"on\" is the same as 16m.",
 "  -fs number[\"k\"|\"m\"]",
@@ -2020,6 +2128,13 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Like -cp_rx but trying to restore timestamps and ownership.",
 "  -paste_in iso_rr_path disk_path byte_offset byte_count",
 "                 Copy ISO file content into a byte interval of a disk file.",
+"  -concat \"append\"|\"overwrite\" target iso_rr_path [***]",
+"                 Write content of iso_rr_paths into disk file target. Target",
+"                 may be a disk_path or \"-\" for writing to standard output.",
+"  -concat \"pipe\" lim prog [args [...]] lim iso_rr_path [***]",
+"                 Start prog with given args and write content of iso_rr_paths",
+"                 into its standard input. prog must at least contain one '/'.",
+"                 lim may be any word that is not among prog and args.",
 "  -mount drive entity id path",
 "                 Like -mount_cmd but actually performing that command if",
 "                 not setuid or setgid is active.",
@@ -2076,12 +2191,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -page len width  Prompt user after len output lines (0=no prompt).",
 "              width (default 80) can adjust line number computation",
 "              to the output terminal's line width.",
-#ifdef Xorriso_with_readlinE
+#ifdef Xorriso_with_line_editoR
 "  -use_readline \"on\"|\"off\"",
 "              Whether to use libreadline for dialog if available.",
 "  -history text  Copy text into libreadline history. This command",
 "              itself is not copied to the history list.",
-#endif /* Xorriso_with_readlinE */
+#endif /* Xorriso_with_line_editoR */
 "  -sh_style_result \"on\"|\"off\"",
 "              If \"on\" do not wrap file addresses in quotation marks with",
 "                -pwd -pwdx -ls -lsd -lsl -lsdl -lsx -lsdx -lslx -lsdlx",
