@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -33,6 +33,10 @@
 /* Option -pacifier */
 int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag)
 {
+
+#define Xorriso_pacifier_min_intvL 0.1
+#define Xorriso_pacifier_max_intvL 60.0
+
  if(strcmp(style, "xorriso")==0 || strcmp(style, "default")==0)
    xorriso->pacifier_style= 0;
  else if(strcmp(style, "mkisofs")==0 || strcmp(style, "genisofs")==0 ||
@@ -41,7 +45,24 @@ int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag)
  else if(strcmp(style, "cdrecord")==0 || strcmp(style, "cdrskin")==0 ||
          strcmp(style, "wodim")==0 || strcmp(style, "xorrecord")==0)
    xorriso->pacifier_style= 2;
- else {
+
+ else if(strncmp(style, "interval=", 9) == 0) {
+   sscanf(style + 9, "%lf", &(xorriso->pacifier_interval));
+   if(xorriso->pacifier_interval < Xorriso_pacifier_min_intvL) {
+     sprintf(xorriso->info_text,
+             "-pacifier: interval='%s' is too low. Min: %f",
+             style, Xorriso_pacifier_min_intvL);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+     xorriso->pacifier_interval= Xorriso_pacifier_min_intvL;
+   } else if(xorriso->pacifier_interval > Xorriso_pacifier_max_intvL) {
+     sprintf(xorriso->info_text,
+             "-pacifier: interval='%s' is too high. Max: %f",
+             style, Xorriso_pacifier_max_intvL);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+     xorriso->pacifier_interval= Xorriso_pacifier_max_intvL;
+   }
+
+ } else {
    sprintf(xorriso->info_text, "-pacifier: unknown behavior code '%s'", style);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
    return(0);
@@ -395,8 +416,10 @@ int Xorriso_option_pwdi(struct XorrisO *xorriso, int flag)
 {
  sprintf(xorriso->info_text,"current working directory in ISO image:\n");
  Xorriso_info(xorriso,0);
- Text_shellsafe(xorriso->wdi, xorriso->result_line, 0);
- strcat(xorriso->result_line, "/\n");
+ Xorriso_esc_filepath(xorriso, xorriso->wdi, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdi[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  return(1);
 }
@@ -407,7 +430,10 @@ int Xorriso_option_pwdx(struct XorrisO *xorriso, int flag)
 {
  sprintf(xorriso->info_text,"current working directory on hard disk:\n");
  Xorriso_info(xorriso,0);
- sprintf(xorriso->result_line,"%s/\n",xorriso->wdx);
+ Xorriso_esc_filepath(xorriso, xorriso->wdx, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdx[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  return(1);
 }
@@ -465,6 +491,28 @@ int Xorriso_option_report_about(struct XorrisO *xorriso, char *in_severity,
 }
 
 
+/* Command -report_el_torito */
+int Xorriso_option_report_el_torito(struct XorrisO *xorriso,
+                                    char *form, int flag)
+{
+ int ret;
+
+ ret= Xorriso_report_system_area(xorriso, form, 1);
+ return(ret);
+}
+
+
+/* Command -report_system_area */
+int Xorriso_option_report_system_area(struct XorrisO *xorriso,
+                                      char *form, int flag)
+{
+ int ret;
+
+ ret= Xorriso_report_system_area(xorriso, form, 0);
+ return(ret);
+}
+
+
 /* Option -return_with */
 int Xorriso_option_return_with(struct XorrisO *xorriso, char *in_severity,
                                int exit_value, int flag)
@@ -1202,6 +1250,23 @@ ex:;
 }
 
 
+/* Option -sh_style_result */
+int Xorriso_option_sh_style_result(struct XorrisO *xorriso, char *mode,
+                                   int flag)
+{
+ if(strcmp(mode, "off") == 0) {
+   xorriso->sh_style_result= 0;
+ } else if(strcmp(mode, "on") == 0) {
+   xorriso->sh_style_result= 1;
+ } else {
+   sprintf(xorriso->info_text, "-sh_style_result: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1); 
+}
+
+
 /* Option -signal_handling */
 /* @param flag bit0= prescan mode: do not yet install the eventual handler
                      else: when switching from other modes to "off":
@@ -1275,19 +1340,34 @@ int Xorriso_option_sleep(struct XorrisO *xorriso, char *duration, int flag)
 }
 
 
-/* Option -speed */
+/* Commands -speed , -read_speed */
+/* @param flag bit0= -read_speed rather than -speed
+*/
 int Xorriso_option_speed(struct XorrisO *xorriso, char *speed, int flag)
 {
- int is_cd= 1, unit_found= 0, ret, profile_number;
- double num;
+ int is_cd= 1, unit_found= 0, ret, profile_number, intspeed= 1;
+ double num= -2.0;
  char *cpt, profile_name[80];
 
- if(speed[0]==0 || strcmp(speed, "any")==0) {
-   xorriso->speed= 0; /* full speed */
+ if(speed[0] == 0 || strcmp(speed, "any") == 0 || strcmp(speed, "max") == 0) {
+   intspeed= 0;
+ } else if(strcmp(speed, "min") == 0) {
+   intspeed= -1;
+ } else if(strcmp(speed, "none") == 0) {
+   intspeed= -2;
+ } else {
+   sscanf(speed,"%lf",&num);
+   if(num <= 0)
+     intspeed= num;
+ }
+ if(intspeed <= 0) {
+   if(flag & 1)
+     xorriso->read_speed= intspeed;
+   else
+     xorriso->write_speed= intspeed;
    return(1);
  }
 
- sscanf(speed,"%lf",&num);
  for(cpt= speed+strlen(speed)-1; cpt>=speed; cpt--)
    if(isdigit(*cpt) || *cpt=='.')
  break;
@@ -1314,7 +1394,8 @@ dvd_speed:;
 bd_speed:;
    num*= 4495.625;
  } else {
-   ret= Xorriso_get_profile(xorriso, &profile_number, profile_name, 2);
+   ret= Xorriso_get_profile(xorriso, &profile_number, profile_name,
+                            2 * !(flag & 1));
    is_cd= (ret==2);
    if(is_cd)
      goto cd_speed;
@@ -1330,9 +1411,13 @@ bd_speed:;
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
    return(0);
  }
- xorriso->speed= num;
- if(xorriso->speed<num)
-   xorriso->speed++;
+ intspeed= num;
+ if(intspeed < num)
+   intspeed++;
+ if(flag & 1)
+   xorriso->read_speed= intspeed;
+ else
+   xorriso->write_speed= intspeed;
  return(1);
 }
 
@@ -1382,26 +1467,28 @@ int Xorriso_option_status_history_max(struct XorrisO *xorriso, int num,
 }
 
 
-/* Option -stdio_sync "on"|"off"|size */
+/* Option -stdio_sync "on"|"off"|"end"|size */
 int Xorriso_option_stdio_sync(struct XorrisO *xorriso, char *rythm, int flag)
 {
  double num;
 
- if(strcmp(rythm, "default") == 0 || strcmp(rythm, "on") == 0)
+ if(strcmp(rythm, "default") == 0 || strcmp(rythm, "on") == 0) {
    num= 0;
- if(strcmp(rythm, "off") == 0)
+ } else if(strcmp(rythm, "off") == 0) {
    num= -1;
- else
-   num = Scanf_io_size(rythm, 0);
- if(num > 0)
-   num /= 2048;
- if(num != -1 && num != 0 && (num < 32 || num > 512 * 1024)) {
+ } else if(strcmp(rythm, "end") == 0) {
+   num= 1;
+ } else {
+   num = Scanf_io_size(rythm, 0) / 2048;
+   if(num < 32 || num > 512 * 1024) {
      sprintf(xorriso->info_text,
-             "-stdio_sync : Bad size. Acceptable are -1, 0, 32k ... 1g");
+           "-stdio_sync : Bad size. Use: 64k to 1g, \"on\", \"off\", \"end\"");
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
      return(0);
- } else
-   xorriso->stdio_sync= num;
+   }
+ }
+ xorriso->stdio_sync= num;
+ xorriso->stdio_sync_is_default= 0;
  return(1);
 }
 
@@ -1804,6 +1891,8 @@ int Xorriso_option_use_readline(struct XorrisO *xorriso, char *mode, int flag)
 /* Option -version */
 int Xorriso_option_version(struct XorrisO *xorriso, int flag)
 {
+ const char *license_text;
+
  sprintf(xorriso->result_line, "%sxorriso %d.%d.%d%s\n",
 #ifdef Xorriso_GNU_xorrisO
          "GNU ",
@@ -1816,7 +1905,7 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag)
  sprintf(xorriso->result_line,
 "ISO 9660 Rock Ridge filesystem manipulator and CD/DVD/BD burn program\n");
  sprintf(xorriso->result_line+strlen(xorriso->result_line),
-"Copyright (C) 2013, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
+"Copyright (C) 2014, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
  Xorriso_result(xorriso, 0);
  sprintf(xorriso->result_line,
          "xorriso version   :  %d.%d.%d%s\n",
@@ -1831,14 +1920,29 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag)
  Xorriso_report_lib_versions(xorriso, 0);
 
 #ifdef Xorriso_GNU_xorrisO
- sprintf(xorriso->result_line,
-"License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n");
+
+ license_text= "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.";
+ sprintf(xorriso->result_line, "%s\n", license_text);
  Xorriso_result(xorriso, 0);
  sprintf(xorriso->result_line,
 "This is free software: you are free to change and redistribute it.\n");
  Xorriso_result(xorriso, 0);
+
 #else
- sprintf(xorriso->result_line, "Provided under GNU GPL version 2 or later.\n");
+
+ license_text= "Provided under GNU GPL version 2 or later.";
+
+#ifdef Xorriso_with_readlinE
+ {
+   const char *readline_license;
+   readline_license= Xorriso__readline_license(0);
+   if(strcmp(readline_license, "GPLv3+") == 0)
+     license_text=
+      "Provided under GNU GPL version 3 or later, due to libreadline license.";
+ }
+#endif /* Xorriso_with_readlinE */
+
+ sprintf(xorriso->result_line, "%s\n", license_text);
  Xorriso_result(xorriso, 0);
 #endif /* ! Xorriso_GNU_xorrisO */
 
@@ -2036,7 +2140,9 @@ int Xorriso_option_zisofs(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->zlib_level= num;
 
    } else if(strncmp(cpt, "ziso_used=", 10) == 0 ||
-             strncmp(cpt, "osiz_used=", 10) == 0) {
+             strncmp(cpt, "osiz_used=", 10) == 0 ||
+             strncmp(cpt, "gzip_used=", 10) == 0 ||
+             strncmp(cpt, "gunzip_used=", 12) == 0) {
      /* (ignored info from -status) */;
 
    } else if(strncmp(cpt, "block_size=", 11)==0) {
