@@ -96,7 +96,7 @@ int Xorriso_opt_args(struct XorrisO *xorriso, char *cmd,
  else
    do_expand= (xorriso->do_iso_rr_pattern==1 && !(flag&4)) || (flag & 512);
  if(flag&256) {
-   if(*optv<argv || *optv>=argv+argc)
+   if(*optv < argv || (*optv >= argv + argc && argc > 0))
      Sfile_destroy_argv(optc, optv, 0);
    return(1);
  }
@@ -197,14 +197,24 @@ ex:;
 }
 
 
+/* @param flag bit0= get eternal problem status
+*/
 int Xorriso_get_problem_status(struct XorrisO *xorriso, char severity[80],
                                int flag)
 {
- strcpy(severity, xorriso->problem_status_text);
- return(xorriso->problem_status);
+ if(flag & 1) {
+   strcpy(severity, xorriso->eternal_problem_status_text);
+   return(xorriso->eternal_problem_status);
+ } else {
+   strcpy(severity, xorriso->problem_status_text);
+   return(xorriso->problem_status);
+ }
 }
 
 
+/* @param flag bit0= set eternal problem status to severity,
+                     and set problem status to ALL
+*/
 int Xorriso_set_problem_status(struct XorrisO *xorriso, char *severity, 
                                int flag)
 {
@@ -237,9 +247,15 @@ int Xorriso_set_problem_status(struct XorrisO *xorriso, char *severity,
 
 #endif /* Xorriso_fetch_with_msg_queueS */
 
- xorriso->problem_status= sev;
- strcpy(xorriso->problem_status_text, sev_text);
- if(sev > xorriso->eternal_problem_status) {
+ if(flag & 1) {
+   strcpy(xorriso->problem_status_text, "ALL");
+   Xorriso__text_to_sev(xorriso->problem_status_text,
+                        &(xorriso->problem_status), 0);
+ } else {
+   xorriso->problem_status= sev;
+   strcpy(xorriso->problem_status_text, sev_text);
+ }
+ if(sev > xorriso->eternal_problem_status || (flag & 1)) {
    xorriso->eternal_problem_status= sev;
    strcpy(xorriso->eternal_problem_status_text, sev_text);
  }
@@ -499,8 +515,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     ""
  };
  static char arg1_commands[][40]= {
-    "abort_on","acl","add_plainly","application_id","auto_charset",
-    "abstract_file",
+    "abort_on","acl","add_plainly","application_id","application_use",
+    "auto_charset","abstract_file",
     "backslash_codes","blank","biblio_file",
     "calm_drive","cd","cdi","cdx","changes_pending","charset",
     "close","close_damaged",
@@ -515,9 +531,10 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "pacifier","padding","path_list","pathspecs","pkt_output",
     "preparer_id","print","print_info","print_mark","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
-    "reassure","report_about","rockridge",
+    "read_speed","reassure","report_about",
+    "report_el_torito","report_system_area","rockridge",
     "rom_toc_scan","rr_reloc_dir","scsi_log",
-    "session_log","signal_handling","sleep",
+    "session_log","sh_style_result","signal_handling","sleep",
     "speed","split_size","status","status_history_max",
     "stdio_sync","stream_recording","system_id","temp_mem_limit","toc_of",
     "uid","unregister_filter","use_readline","volid","volset_id",
@@ -538,7 +555,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     ""
  };
  static char arg4_commands[][40]= {
-    "cut_out","extract_cut","mount","mount_cmd","paste_in","session_string",
+    "cut_out","extract_cut","mount","mount_cmd","named_pipe_loop",
+    "paste_in","session_string",
     ""
  };
  static char argn_commands[][40]= {
@@ -546,7 +564,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "check_md5","check_md5_r","check_media","check_media_defaults",
     "chgrp","chgrpi","chgrp_r","chgrp_ri","chmod","chmodi",
     "chmod_r","chmod_ri","chown","chowni","chown_r","chown_ri",
-    "compare_l","cp_clone","cp_rax","cp_rx","cpr","cpri", "cpax","cpx",
+    "compare_l","concat","cp_clone","cp_rax","cp_rx","cpr","cpri","cpax","cpx",
     "du","dui","dus","dusi","dux","dusx","external_filter","extract_l",
     "file_size_limit","find","findi","finds","findx",
     "getfacl","getfacli","getfacl_r","getfacl_ri",
@@ -641,13 +659,15 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
    "errfile_log", "session_log", "scsi_log",
    "options_from_file", "list_delimiter",
    "print", "print_info", "print_mark", "prompt", "sleep",
+   "sh_style_result",
 
    "* Drive and media related inquiry actions (1):",
    "devices", "device_links",
    "mount_opts", "mount_cmd", "session_string",
 
    "* Influencing the behavior of image loading:",
-   "load", "displacement", "drive_class", "assert_volid", "in_charset",
+   "read_speed", "load", "displacement",
+   "drive_class", "assert_volid", "in_charset",
    "auto_charset", "hardlinks", "acl", "xattr", "md5", "for_backup",
    "disk_dev_ino", "rom_toc_scan", "calm_drive", "ban_stdio_write",
    "early_stdio_test", "data_cache_size",
@@ -660,7 +680,7 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
 
    "* Drive and media related inquiry actions (2):",
    "list_profiles", "list_formats", "list_speeds",
-   "toc", "toc_of", "pvd_info",
+   "toc", "toc_of", "pvd_info", "report_system_area", "report_el_torito",
 
    "* Settings for file insertion:",
    "file_size_limit", "not_mgt", "not_paths", "not_leaf", "not_list",
@@ -698,15 +718,15 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
 
    "* osirrox ISO-to-disk restore options:",
    "osirrox", "extract", "extract_single", "extract_l", "extract_cut",
-   "cpx", "cpax", "cp_rx", "cp_rax", "paste_in",
+   "cpx", "cpax", "cp_rx", "cp_rax", "paste_in", "concat",
    "mount",
 
    "* Settings for result writing:",
    "rockridge", "joliet", "hfsplus","compliance", "rr_reloc_dir",
    "volid", "volset_id", "publisher",
    "application_id", "system_id", "volume_date", "copyright_file",
-   "abstract_file", "biblio_file", "preparer_id", "out_charset",
-   "read_mkisofsrc",
+   "abstract_file", "biblio_file", "preparer_id", "application_use",
+   "out_charset", "read_mkisofsrc",
    "uid", "gid", "zisofs", "speed", "stream_recording", "dvd_obs",
    "stdio_sync", "dummy", "fs", "close", "padding", "write_type",
    "grow_blindly", "pacifier", "scdbackup_tag",
@@ -741,7 +761,7 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
    "dialog", "page", "use_readline", "reassure",
 
    "* Support for frontend programs via stdin and stdout (3):",
-   "launch_frontend",
+   "launch_frontend", "named_pipe_loop",
 
    "* Scripting, dialog and program control features (3):",
    "rollback_end", "end",
@@ -902,7 +922,7 @@ return:
 {
  int ret, was_dashed, end_ret, num1, num2, cmd_data_size= 2 * SfileadrL;
  int mem_idx, arg_count, i;
- char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2, *arg3;
+ char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2, *arg3, *arg4;
  
  Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
 
@@ -1018,6 +1038,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_application_id(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"application_use") == 0) {
+   (*idx)++;
+   ret= Xorriso_option_application_use(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"as")==0) {
    ret= Xorriso_option_as(xorriso, argc, argv, idx, 0);
 
@@ -1141,6 +1165,9 @@ next_command:;
    (*idx)++;
    Xorriso_option_compliance(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"concat") == 0) {
+   ret= Xorriso_option_concat(xorriso, argc, argv, idx, 0);
+
  } else if(strcmp(cmd,"copyright_file")==0) { 
    (*idx)++;
    Xorriso_option_copyright_file(xorriso, arg1, 0);
@@ -1460,6 +1487,16 @@ next_command:;
  } else if(strcmp(cmd,"mkdir")==0 || strcmp(cmd,"mkdiri")==0) {
    ret= Xorriso_option_mkdiri(xorriso, argc, argv, idx, 0);
 
+ } else if(strcmp(cmd,"named_pipe_loop")==0) {
+   if((*idx) + 3 < argc)
+     arg4= argv[(*idx) + 3];
+   else
+     arg4= "";
+   (*idx)+= 4;
+   ret= Xorriso_option_named_pipe_loop(xorriso, arg1, arg2, arg3, arg4, 0);
+   if(ret == 3)
+     goto ex;
+
  } else if(strcmp(cmd,"no_rc")==0) {
    ret= Xorriso_option_no_rc(xorriso, 0);
 
@@ -1595,6 +1632,10 @@ next_command:;
  } else if(strcmp(cmd,"read_mkisofsrc")==0) {
    ret= Xorriso_option_read_mkisofsrc(xorriso, 0);
 
+ } else if(strcmp(cmd,"read_speed")==0) {
+   (*idx)++;
+   ret= Xorriso_option_speed(xorriso, arg1, 1);
+
  } else if(strcmp(cmd,"reassure")==0) {
    (*idx)++;
    ret= Xorriso_option_reassure(xorriso, arg1, 0);
@@ -1603,6 +1644,14 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_report_about(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"report_el_torito")==0) {
+   (*idx)++;
+   ret= Xorriso_option_report_el_torito(xorriso, arg1, 0);
+
+ } else if(strcmp(cmd,"report_system_area")==0) {
+   (*idx)++;
+   ret= Xorriso_option_report_system_area(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"return_with")==0) {
    (*idx)+= 2;
    num2= 0;
@@ -1698,6 +1747,10 @@ next_command:;
    ret= Xorriso_option_set_filter(xorriso, "", argc, argv, idx,
                                   (strcmp(cmd,"show_stream_r")==0) | 2 | 4);
 
+ } else if(strcmp(cmd,"sh_style_result")==0) {
+   (*idx)++;
+   ret= Xorriso_option_sh_style_result(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"signal_handling")==0) {
    (*idx)++;
    ret= Xorriso_option_signal_handling(xorriso, arg1, 0);
@@ -2086,6 +2139,26 @@ ex:;
 }
 
 
+/* @return  1=replaced , 2=not replaced , <=0 = error
+*/
+int Xorriso_replace_arg_by_bsl(struct XorrisO *xorriso, char **arg,
+                               char **argpt, int flag)
+{
+ int ret, eaten, l;
+
+ if(!(xorriso->bsl_interpretation & 16))
+   return(2);
+ l= strlen(*argpt);
+ Xorriso_free_meM(*arg);
+ Xorriso_alloc_meM(*arg, char, l + 1);
+ strcpy(*arg, *argpt);
+ *argpt= *arg;
+ ret= Sfile_bsl_interpreter(*arg, l, &eaten, 0);
+ex:;
+ return(ret);
+}
+
+
 int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
                          int flag)
 /*
@@ -2100,14 +2173,16 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
 */
 {
  int i, ret, was_dashed, num2, arg_count;
- int advice, mem_add_plainly, error_seen= 0;
+ int advice, mem_add_plainly, error_seen= 0, mem_bsl;
  int was_report_about= 0, was_abort_on= 0, was_return_with= 0;
  int was_signal_handling= 0, was_scsi_log= 0, cmd_data_size= 5 * SfileadrL;
  char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
+ char *arg1_data= NULL, *arg2_data= NULL;
  char mem_list_delimiter[81];
 
  strcpy(mem_list_delimiter, xorriso->list_delimiter);
  mem_add_plainly= xorriso->add_plainly;
+ mem_bsl= xorriso->bsl_interpretation;
 
  Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
 
@@ -2121,11 +2196,19 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
      {ret= -1; goto ex;}
 
    arg1= "";
-   if(i+1<argc)
-     arg1= argv[i+1];
+   if(i + 1 < argc) {
+     arg1= argv[i + 1];
+     ret= Xorriso_replace_arg_by_bsl(xorriso, &arg1_data, &arg1, 0);
+     if(ret <= 0)
+       goto ex;
+   }
    arg2= "";
-   if(i+2<argc)
-     arg2= argv[i+2];
+   if(i + 2 < argc) {
+     arg2= argv[i + 2];
+     ret= Xorriso_replace_arg_by_bsl(xorriso, &arg2_data, &arg2, 0);
+     if(ret <= 0)
+       goto ex;
+   }
    if(i>1)
      xorriso->did_something_useful= 1;
    if(i==1 && argc==2) {
@@ -2255,6 +2338,12 @@ protect_stdout:;
    } else if(strcmp(original_cmd, "-x") == 0) {
      xorriso->arrange_args= 1;
 
+   } else if(strcmp(cmd, "backslash_codes") == 0) {
+     i++;
+     ret= Xorriso_option_backslash_codes(xorriso, arg1, 0);
+     if(ret <= 0)
+       error_seen= 1;
+
    } else {
      ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
      if(ret == 1) {
@@ -2272,6 +2361,9 @@ protect_stdout:;
 ex:;
  strcpy(xorriso->list_delimiter, mem_list_delimiter);
  xorriso->add_plainly= mem_add_plainly;
+ xorriso->bsl_interpretation= mem_bsl;
+ Xorriso_free_meM(arg1_data);
+ Xorriso_free_meM(arg2_data);
  Xorriso_free_meM(cmd_data);
  if(error_seen && ret > 0) {
    advice= Xorriso_eval_problem_status(xorriso, 0, 0);
@@ -2472,11 +2564,74 @@ int Xorriso_make_return_value(struct XorrisO *xorriso, int flag)
 int Xorriso_program_arg_bsl(struct XorrisO *xorriso, int argc, char ***argv,
                             int flag)
 {
- int ret;
+ int i, ret, eaten, bsl_mem, params_to_come= 0, cmd_data_size= 5 * SfileadrL;
+ int next_is_backslash_codes= 0, next_is_list_delimiter= 0;
+ char **new_argv= NULL, *cmd, *cmd_data= NULL;
+ char mem_list_delimiter[81];
 
- if(!(xorriso->bsl_interpretation & 16))
-   return(1);
- ret= Sfile_argv_bsl(argc, argv, 0);
+ strcpy(mem_list_delimiter, xorriso->list_delimiter);
+ bsl_mem= xorriso->bsl_interpretation;
+ if(argc <= 0)
+   return(0);
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+ new_argv= (char **) Smem_malloC(argc * sizeof(char *));
+ if(new_argv == NULL)
+   return(-1); 
+ for(i= 0; i < argc; i++) {
+   new_argv[i]= strdup((*argv)[i]);
+   if(new_argv[i] == NULL)
+     {ret= -1; goto ex;}
+   if(i == 0)
+ continue;
+   if(xorriso->bsl_interpretation & 16) {
+     ret= Sfile_bsl_interpreter(new_argv[i], strlen(new_argv[i]), &eaten, 0);
+     if(ret <= 0)
+       goto ex;
+   }
+   if(params_to_come == 0) {
+     ret= Xorriso_normalize_command(xorriso, new_argv[i], i,
+                                    cmd_data, cmd_data_size, &cmd, 0);
+     if(ret < 0)
+       goto ex;
+     if(strcmp(cmd, "backslash_codes") == 0) {
+       params_to_come= 1;
+       next_is_backslash_codes= 1;
+     } else if(strcmp(cmd, "list_delimiter") == 0) {
+       params_to_come= 1;
+       next_is_list_delimiter= 1;
+     } else {
+       ret= Xorriso_count_args(xorriso, argc - i, *argv + i,
+                               &params_to_come, 1);
+       if(ret <= 0)
+         goto ex;
+       if(ret != 1)
+         params_to_come= 0;
+     }
+   } else {
+     params_to_come--;
+     if(next_is_backslash_codes) {
+       next_is_backslash_codes= 0;
+       ret= Xorriso_option_backslash_codes(xorriso, new_argv[i], 0);
+       if(ret <= 0)
+         goto ex;
+     } else if(next_is_list_delimiter) {
+       next_is_list_delimiter= 0;
+       ret= Xorriso_option_list_delimiter(xorriso, new_argv[i], 0);
+       if(ret <= 0)
+         goto ex;
+     }
+   }
+ }
+ ret= 1;
+ex:;
+ Xorriso_free_meM(cmd_data);
+ strcpy(xorriso->list_delimiter, mem_list_delimiter);
+ xorriso->bsl_interpretation= bsl_mem;
+ if(ret <= 0) {
+   if(new_argv != NULL)
+     free((char *) new_argv);
+ } else
+   *argv= new_argv;
  return(ret);
 }
 
@@ -2532,12 +2687,21 @@ int Xorriso_convert_datestring(struct XorrisO *xorriso, char *cmd,
 {
  int ret;
 
+ *t_type= 0;
  if(strcmp(time_type, "a")==0)
    (*t_type)|= 1;
+ else if(strcmp(time_type, "a-c")==0)
+   (*t_type)|= 1 | 256;
  else if(strcmp(time_type, "m")==0)
    (*t_type)|= 4;
+ else if(strcmp(time_type, "m-c")==0)
+   (*t_type)|= 4 | 256;
  else if(strcmp(time_type, "b")==0)
    (*t_type)|= 5;
+ else if(strcmp(time_type, "b-c")==0)
+   (*t_type)|= 5 | 256;
+ else if(strcmp(time_type, "c")==0)
+   (*t_type)|= 2 | 256;
  else {
    sprintf(xorriso->info_text, "%s: Unrecognized type '%s'", cmd, time_type);
    if(!(flag & 1))
@@ -2583,16 +2747,67 @@ int Xorriso_check_temp_mem_limit(struct XorrisO *xorriso, off_t mem, int flag)
 }
 
 
+int Xorriso_wait_child_end(struct XorrisO *xorriso, pid_t child_pid,
+                           int *status, int flag)
+{
+ int ret;
+
+ do {
+   /* try to read and print the reply */;
+   ret= waitpid(child_pid, status, WNOHANG);
+   if(ret == -1) {
+     if(errno != EINTR)
+       return(0);
+   } else if(ret == 0) {
+ continue;
+   } else {
+ break;
+   }
+ } while(1);
+
+ /* >>> interpret *status */;
+
+ return(1);
+}
+
+
+int Xorriso_make_argv_with_null(struct XorrisO *xorriso,
+                                int in_argc, char **in_argv,
+                                int *argc, char ***argv, int flag)
+{
+ int i, ret= 0;
+
+ *argv= NULL;
+ Xorriso_alloc_meM(*argv, char *, in_argc + 1);
+ for(i= 0; i < in_argc; i++) {
+   Xorriso_alloc_meM((*argv)[i], char, strlen(in_argv[i]) + 1);
+   strcpy((*argv)[i], in_argv[i]);
+   *argc= i + 1;
+ }
+ (*argv)[in_argc]= NULL;
+ ret= 1;
+ex:;
+ if(ret <= 0)
+   Sfile_destroy_argv(argc, argv, 0);
+ return(ret);
+}
+
+
 /*
   @param flag bit0= use env_path to find the desired program
+              bit1= use in_argv rather than parsing cmd to words
+              bit2= -reserved-
+              bit3= demand absolute cmd path
 return:
  <=0 : error
  1   : done
 */
-int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
+int Xorriso_execv(struct XorrisO *xorriso, char *cmd,
+                  int in_argc, char **in_argv, char *env_path,
+                  int *stdin_pipe, int *stdout_pipe, pid_t *forked_pid,
                   int *status, int flag)
 {
- int ret, argc= 0;
+ int ret, argc= 0, has_slash;
  char **argv= NULL, *pathlist= NULL, *cpt, *npt, *prog= NULL;
  pid_t child_pid;
  struct stat stbuf;
@@ -2601,14 +2816,26 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
 
  wait3(NULL,WNOHANG,NULL); /* just to remove any old dead child */
 
- ret= Sfile_make_argv("", cmd, &argc, &argv, 1|4|128);
+ if(flag & 2) {
+   ret= Xorriso_make_argv_with_null(xorriso, in_argc, in_argv,
+                                    &argc, &argv, 0);
+ } else {
+   ret= Sfile_make_argv("", cmd, &argc, &argv, 1|4|128);
+ }
  if(ret <= 0)
    goto ex;
  if(argc < 1)
    {ret= 0; goto ex;}
 
  strcpy(prog, argv[0]);
- if((flag & 1) && strchr(argv[0], '/') == NULL) {
+ has_slash= (strchr(argv[0], '/') != NULL);
+ if((flag & 8) && !has_slash) {
+   sprintf(xorriso->info_text, "External program path contains no '/': ");
+   Text_shellsafe(argv[0], xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if((flag & 1) && !has_slash) {
      if(env_path == NULL)
        env_path= "/bin:/sbin";
      else if(env_path[0] == 0)
@@ -2645,9 +2872,21 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
    sprintf(xorriso->info_text, "Executing external program ");
    Text_shellsafe(prog, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
-
    Xorriso_destroy(&xorriso, 0); /* reduce memory foot print */
 
+   if(stdin_pipe != NULL) {
+     close(0);
+     if(dup2(stdin_pipe[0], 0) == -1)
+            { perror("dup2(,0)"); exit(1); }
+     close(stdin_pipe[1]); /* unused */
+   }
+   if(stdout_pipe != NULL) {
+     close(1);
+     if(dup2(stdout_pipe[1], 1) == -1)
+            { perror("dup2(,1)"); exit(1); }
+     close(stdout_pipe[0]); /* unused */
+   }
+
    execv(prog, argv); /* should never come back */
    fprintf(stderr,"--- execution of shell command failed:\n");
    fprintf(stderr,"    %s\n",cmd);
@@ -2656,33 +2895,22 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
 
 
          /* this is the original process waiting for child to exit */
- do {
-   /* try to read and print the reply */;
-   ret= waitpid(child_pid,status,WNOHANG);
-   if(ret==-1) {
-     if(errno!=EINTR)
-       ret= 0; goto ex;
-   } else if(ret==0) {
 
-#ifdef NIX
+ if(stdin_pipe != NULL)
+   close(stdin_pipe[0]); /* unused */
+ if(stdout_pipe != NULL)
+   close(stdout_pipe[1]); /* unused */
+ if(stdin_pipe != NULL || stdout_pipe != NULL) {
+   /* Pipes need to be fed by the caller who later calls Xorriso_wait_child_end
+      with *forked_pid as child_pid.
+   */
+   *forked_pid= child_pid;
+   {ret= 1; goto ex;}
+ }
 
-     /* >>> An interrupt key would be nice. */
-
-     if((flag&4)) {
-       ret= Asqueue_event_is_pending(agent->queue,0,0);
-       if(ret>0) {
-         Asagent_stderr(agent,"--- shell command interrupted",1);
-         kill(child_pid,SIGTERM);
-         ret= 2; goto ex;
-       }
-     }
-#endif /* NIX */
-
- continue;
-   } else {
- break;
-   }
- } while(1);
+ ret= Xorriso_wait_child_end(xorriso, child_pid, status, 0);
+ if(ret <= 0)
+   goto ex;
  ret= 1;
 ex:
  Sfile_make_argv("", "", &argc, &argv, 2);
@@ -2692,6 +2920,43 @@ ex:
 }
 
 
+/* @param flag bit0= use env_path to find the desired program
+               bit1= use in_argv rather than parsing cmd to words
+               bit2= "r" rather than "w"
+               bit3= demand absolute cmd path
+               bit4= override any restriction on external filters
+*/
+int Xorriso_pipe_open(struct XorrisO *xorriso, char *purpose, char *cmd,
+                      int in_argc, char **in_argv, char *env_path,
+                      int *fd, pid_t *forked_pid, int flag)
+{
+ int fp_pipe[2], *stdin_pipe= NULL, *stdout_pipe= NULL, status, ret;
+
+ *fd= -1;
+
+ if(!(flag & 16)) {
+   ret= Xorriso_external_filter_banned(xorriso, purpose, 0);
+   if(ret)
+     return(0);
+ }
+ if(pipe(fp_pipe) != 0) {
+   sprintf(xorriso->info_text, "Cannot create pipe(2) object");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FATAL", 0);
+   return(0);
+ }
+ if(flag & 4) {
+   stdout_pipe= fp_pipe;
+   *fd= fp_pipe[0];
+ } else {
+   stdin_pipe= fp_pipe;
+   *fd= fp_pipe[1];
+ }
+ ret= Xorriso_execv(xorriso, cmd, in_argc, in_argv, env_path,
+                    stdin_pipe, stdout_pipe, forked_pid, &status, flag & 11);
+ return(ret);
+}
+
+
 /* @param flag bit0= path is a command parameter
 */
 int Xorriso_path_is_excluded(struct XorrisO *xorriso, char *path, int flag)
