@@ -48,6 +48,9 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
 int Xorriso_set_isolinux_options(struct XorrisO *xorriso,
                                  IsoImage *image, int flag);
 
+int Xorriso_make_iso_write_opts(struct XorrisO *xorriso, IsoImage *image,
+                                struct isoburn_imgen_opts *sopts, int flag);
+
 
 #endif /* ! Xorriso_pvt_write_run_includeD */
 
