@@ -68,7 +68,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  3
-#define Xorriso_header_version_micrO  6
+#define Xorriso_header_version_micrO  8
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -1389,6 +1389,11 @@ int Xorriso_option_compare(struct XorrisO *xorriso, char *disk_path,
 /* Command -compliance */
 int Xorriso_option_compliance(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Command -concat */
+/* @since 1.3.8 */
+int Xorriso_option_concat(struct XorrisO *xorriso,
+                          int argc, char **argv, int *idx, int flag);
+
 /* Command -copyright_file */
 /* @since 0.6.0 */
 int Xorriso_option_copyright_file(struct XorrisO *xorriso, char *name,
@@ -1795,6 +1800,14 @@ int Xorriso_option_reassure(struct XorrisO *xorriso, char *mode, int flag);
 int Xorriso_option_report_about(struct XorrisO *xorriso, char *severity, 
                                 int flag);
 
+/* Command -report_el_torito */
+int Xorriso_option_report_el_torito(struct XorrisO *xorriso,
+                                    char *form, int flag);
+
+/* Command -report_system_area */
+int Xorriso_option_report_system_area(struct XorrisO *xorriso,
+                                      char *form, int flag);
+
 /* Command -return_with */
 int Xorriso_option_return_with(struct XorrisO *xorriso, char *severity,
                                int exit_value, int flag);
@@ -1887,7 +1900,7 @@ int Xorriso_option_status(struct XorrisO *xorriso, char *mode, int flag);
 int Xorriso_option_status_history_max(struct XorrisO *xorriso, int num1, 
                                       int flag);
 
-/* Command -stdio_sync "on"|"off"|size */
+/* Command -stdio_sync "on"|"off"|"end"|size */
 int Xorriso_option_stdio_sync(struct XorrisO *xorriso, char *rythm, int flag);
 
 /* Command -stream_recording */
