@@ -192,6 +192,7 @@ struct XorrisO { /* the global context of xorriso */
  int follow_link_limit;
  int resolve_link_rec_count;
  int resolve_link_rec_limit;
+ int do_follow_concat;
  int do_follow_mount;
  int do_global_uid;
  uid_t global_uid;
@@ -363,7 +364,9 @@ struct XorrisO { /* the global context of xorriso */
                              >=16 means yes with number as start LBA  */
 
  int dvd_obs;                 /* DVD write chunk size: 0, 32k or 64k */
+
  int stdio_sync;              /* stdio fsync interval: -1, 0, >=32 */
+ int stdio_sync_is_default;   /* 1= is still default , 0= has been set */
 
  int keep_boot_image;
  char boot_image_cat_path[SfileadrL];
@@ -414,6 +417,7 @@ struct XorrisO { /* the global context of xorriso */
                                                0 = auto (align if bit1)
                                                1 = always align
                                                2 = never align
+                                               3 = align external partitions
                                          bit10-13= System area sub type
                                                With type 0 = MBR:
                                                Gets overridden by bit0 and bit1.
@@ -424,6 +428,7 @@ struct XorrisO { /* the global context of xorriso */
                                                    any other feature which
                                                    needs to have own MBR
                                                    partition entries.
+                                               2 = generic MBR
                                          bit14= Only with System area type 0
                                                GRUB2 boot provisions:
                                                Patch system area at byte 92 to
