@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  3
-#define xorriso_libisoburn_req_micro  6
+#define xorriso_libisoburn_req_micro  8
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -245,7 +245,7 @@ int Xorriso_retry_burn_track(struct XorrisO *xorriso,
 int Xorriso_get_profile(struct XorrisO *xorriso, int *profile_number,
                         char profile_name[80], int flag);
 
-/* @param flag bit0= node_pt is a valid ISO object handle, ignore pathname
+/* @param flag bit0= open IsoNode *node_pt rather than looking up pathname
                bit1= dig out the most original stream for reading
 */
 int Xorriso_iso_file_open(struct XorrisO *xorriso, char *pathname,
@@ -256,6 +256,9 @@ int Xorriso_iso_file_read(struct XorrisO *xorriso, void *stream, char *buf,
 
 int Xorriso_iso_file_close(struct XorrisO *xorriso, void **stream, int flag);
 
+int Xorriso_iso_file_to_fd(struct XorrisO *xorriso, char *path, int fd,
+                           int flag);
+
 /* @param bit0= copy link target properties rather than link properties
 */
 int Xorriso_copy_properties(struct XorrisO *xorriso,
@@ -620,6 +623,11 @@ int Xorriso_pretend_full_disc(struct XorrisO *xorriso, int flag);
 
 int Xorriso_apply_sort_file(struct XorrisO *xorriso, char *path, int flag);
 
+int Xorriso_report_system_area(struct XorrisO *xorriso, char *form, int flag);
+
+int Xorriso_external_filter_banned(struct XorrisO *xorriso, char *purpose,
+                                   int flag);
+
 
 #endif /* Xorrisoburn_includeD */
 
